# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._account_session import AccountSession
from stripe._request_options import RequestOptions
from stripe._stripe_service import StripeService
from typing import List, cast
from typing_extensions import NotRequired, TypedDict


class AccountSessionService(StripeService):
    class CreateParams(TypedDict):
        account: str
        """
        The identifier of the account to create an Account Session for.
        """
        components: "AccountSessionService.CreateParamsComponents"
        """
        Each key of the dictionary represents an embedded component, and each embedded component maps to its configuration (e.g. whether it has been enabled or not).
        """
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """

    class CreateParamsComponents(TypedDict):
        account_onboarding: NotRequired[
            "AccountSessionService.CreateParamsComponentsAccountOnboarding"
        ]
        """
        Configuration for the account onboarding embedded component.
        """
        capital_financing_promotion: NotRequired[
            "AccountSessionService.CreateParamsComponentsCapitalFinancingPromotion"
        ]
        payment_details: NotRequired[
            "AccountSessionService.CreateParamsComponentsPaymentDetails"
        ]
        """
        Configuration for the payment details embedded component.
        """
        payments: NotRequired[
            "AccountSessionService.CreateParamsComponentsPayments"
        ]
        """
        Configuration for the payments embedded component.
        """
        payouts: NotRequired[
            "AccountSessionService.CreateParamsComponentsPayouts"
        ]
        """
        Configuration for the payouts embedded component.
        """

    class CreateParamsComponentsAccountOnboarding(TypedDict):
        enabled: bool
        """
        Whether the embedded component is enabled.
        """
        features: NotRequired[
            "AccountSessionService.CreateParamsComponentsAccountOnboardingFeatures"
        ]
        """
        The list of features enabled in the embedded component.
        """

    class CreateParamsComponentsAccountOnboardingFeatures(TypedDict):
        pass

    class CreateParamsComponentsCapitalFinancingPromotion(TypedDict):
        enabled: bool
        """
        Whether the embedded component is enabled.
        """
        features: NotRequired[
            "AccountSessionService.CreateParamsComponentsCapitalFinancingPromotionFeatures"
        ]
        """
        The list of features enabled in the embedded component.
        """

    class CreateParamsComponentsCapitalFinancingPromotionFeatures(TypedDict):
        pass

    class CreateParamsComponentsPaymentDetails(TypedDict):
        enabled: bool
        """
        Whether the embedded component is enabled.
        """
        features: NotRequired[
            "AccountSessionService.CreateParamsComponentsPaymentDetailsFeatures"
        ]
        """
        The list of features enabled in the embedded component.
        """

    class CreateParamsComponentsPaymentDetailsFeatures(TypedDict):
        capture_payments: NotRequired["bool"]
        """
        Whether to allow capturing and cancelling payment intents. This is `true` by default.
        """
        dispute_management: NotRequired["bool"]
        """
        Whether to allow responding to disputes, including submitting evidence and accepting disputes. This is `true` by default.
        """
        refund_management: NotRequired["bool"]
        """
        Whether to allow sending refunds. This is `true` by default.
        """

    class CreateParamsComponentsPayments(TypedDict):
        enabled: bool
        """
        Whether the embedded component is enabled.
        """
        features: NotRequired[
            "AccountSessionService.CreateParamsComponentsPaymentsFeatures"
        ]
        """
        The list of features enabled in the embedded component.
        """

    class CreateParamsComponentsPaymentsFeatures(TypedDict):
        capture_payments: NotRequired["bool"]
        """
        Whether to allow capturing and cancelling payment intents. This is `true` by default.
        """
        dispute_management: NotRequired["bool"]
        """
        Whether to allow responding to disputes, including submitting evidence and accepting disputes. This is `true` by default.
        """
        refund_management: NotRequired["bool"]
        """
        Whether to allow sending refunds. This is `true` by default.
        """

    class CreateParamsComponentsPayouts(TypedDict):
        enabled: bool
        """
        Whether the embedded component is enabled.
        """
        features: NotRequired[
            "AccountSessionService.CreateParamsComponentsPayoutsFeatures"
        ]
        """
        The list of features enabled in the embedded component.
        """

    class CreateParamsComponentsPayoutsFeatures(TypedDict):
        edit_payout_schedule: NotRequired["bool"]
        """
        Whether to allow payout schedule to be changed. Default `true` when Stripe owns Loss Liability, default `false` otherwise.
        """
        instant_payouts: NotRequired["bool"]
        """
        Whether to allow creation of instant payouts. Default `true` when Stripe owns Loss Liability, default `false` otherwise.
        """
        standard_payouts: NotRequired["bool"]
        """
        Whether to allow creation of standard payouts. Default `true` when Stripe owns Loss Liability, default `false` otherwise.
        """

    def create(
        self,
        params: "AccountSessionService.CreateParams",
        options: RequestOptions = {},
    ) -> AccountSession:
        """
        Creates a AccountSession object that includes a single-use token that the platform can use on their front-end to grant client-side API access.
        """
        return cast(
            AccountSession,
            self._request(
                "post",
                "/v1/account_sessions",
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "AccountSessionService.CreateParams",
        options: RequestOptions = {},
    ) -> AccountSession:
        """
        Creates a AccountSession object that includes a single-use token that the platform can use on their front-end to grant client-side API access.
        """
        return cast(
            AccountSession,
            await self._request_async(
                "post",
                "/v1/account_sessions",
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._request_options import RequestOptions
from stripe._stripe_object import StripeObject
from typing import ClassVar, List, Optional, cast
from typing_extensions import Literal, NotRequired, TypedDict, Unpack


class Secret(CreateableAPIResource["Secret"], ListableAPIResource["Secret"]):
    """
    Secret Store is an API that allows Stripe Apps developers to securely persist secrets for use by UI Extensions and app backends.

    The primary resource in Secret Store is a `secret`. Other apps can't view secrets created by an app. Additionally, secrets are scoped to provide further permission control.

    All Dashboard users and the app backend share `account` scoped secrets. Use the `account` scope for secrets that don't change per-user, like a third-party API key.

    A `user` scoped secret is accessible by the app backend and one specific Dashboard user. Use the `user` scope for per-user secrets like per-user OAuth tokens, where different users might have different permissions.

    Related guide: [Store data between page reloads](https://stripe.com/docs/stripe-apps/store-auth-data-custom-objects)
    """

    OBJECT_NAME: ClassVar[Literal["apps.secret"]] = "apps.secret"

    class Scope(StripeObject):
        type: Literal["account", "user"]
        """
        The secret scope type.
        """
        user: Optional[str]
        """
        The user ID, if type is set to "user"
        """

    class CreateParams(RequestOptions):
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """
        expires_at: NotRequired["int"]
        """
        The Unix timestamp for the expiry time of the secret, after which the secret deletes.
        """
        name: str
        """
        A name for the secret that's unique within the scope.
        """
        payload: str
        """
        The plaintext secret value to be stored.
        """
        scope: "Secret.CreateParamsScope"
        """
        Specifies the scoping of the secret. Requests originating from UI extensions can only access account-scoped secrets or secrets scoped to their own user.
        """

    class CreateParamsScope(TypedDict):
        type: Literal["account", "user"]
        """
        The secret scope type.
        """
        user: NotRequired["str"]
        """
        The user ID. This field is required if `type` is set to `user`, and should not be provided if `type` is set to `account`.
        """

    class DeleteWhereParams(RequestOptions):
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """
        name: str
        """
        A name for the secret that's unique within the scope.
        """
        scope: "Secret.DeleteWhereParamsScope"
        """
        Specifies the scoping of the secret. Requests originating from UI extensions can only access account-scoped secrets or secrets scoped to their own user.
        """

    class DeleteWhereParamsScope(TypedDict):
        type: Literal["account", "user"]
        """
        The secret scope type.
        """
        user: NotRequired["str"]
        """
        The user ID. This field is required if `type` is set to `user`, and should not be provided if `type` is set to `account`.
        """

    class FindParams(RequestOptions):
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """
        name: str
        """
        A name for the secret that's unique within the scope.
        """
        scope: "Secret.FindParamsScope"
        """
        Specifies the scoping of the secret. Requests originating from UI extensions can only access account-scoped secrets or secrets scoped to their own user.
        """

    class FindParamsScope(TypedDict):
        type: Literal["account", "user"]
        """
        The secret scope type.
        """
        user: NotRequired["str"]
        """
        The user ID. This field is required if `type` is set to `user`, and should not be provided if `type` is set to `account`.
        """

    class ListParams(RequestOptions):
        ending_before: NotRequired["str"]
        """
        A cursor for use in pagination. `ending_before` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, starting with `obj_bar`, your subsequent call can include `ending_before=obj_bar` in order to fetch the previous page of the list.
        """
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """
        limit: NotRequired["int"]
        """
        A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
        """
        scope: "Secret.ListParamsScope"
        """
        Specifies the scoping of the secret. Requests originating from UI extensions can only access account-scoped secrets or secrets scoped to their own user.
        """
        starting_after: NotRequired["str"]
        """
        A cursor for use in pagination. `starting_after` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with `obj_foo`, your subsequent call can include `starting_after=obj_foo` in order to fetch the next page of the list.
        """

    class ListParamsScope(TypedDict):
        type: Literal["account", "user"]
        """
        The secret scope type.
        """
        user: NotRequired["str"]
        """
        The user ID. This field is required if `type` is set to `user`, and should not be provided if `type` is set to `account`.
        """

    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    deleted: Optional[bool]
    """
    If true, indicates that this secret has been deleted
    """
    expires_at: Optional[int]
    """
    The Unix timestamp for the expiry time of the secret, after which the secret deletes.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    name: str
    """
    A name for the secret that's unique within the scope.
    """
    object: Literal["apps.secret"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    payload: Optional[str]
    """
    The plaintext secret value to be stored.
    """
    scope: Scope

    @classmethod
    def create(cls, **params: Unpack["Secret.CreateParams"]) -> "Secret":
        """
        Create or replace a secret in the secret store.
        """
        return cast(
            "Secret",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["Secret.CreateParams"]
    ) -> "Secret":
        """
        Create or replace a secret in the secret store.
        """
        return cast(
            "Secret",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def delete_where(
        cls, **params: Unpack["Secret.DeleteWhereParams"]
    ) -> "Secret":
        """
        Deletes a secret from the secret store by name and scope.
        """
        return cast(
            "Secret",
            cls._static_request(
                "post",
                "/v1/apps/secrets/delete",
                params=params,
            ),
        )

    @classmethod
    async def delete_where_async(
        cls, **params: Unpack["Secret.DeleteWhereParams"]
    ) -> "Secret":
        """
        Deletes a secret from the secret store by name and scope.
        """
        return cast(
            "Secret",
            await cls._static_request_async(
                "post",
                "/v1/apps/secrets/delete",
                params=params,
            ),
        )

    @classmethod
    def find(cls, **params: Unpack["Secret.FindParams"]) -> "Secret":
        """
        Finds a secret in the secret store by name and scope.
        """
        return cast(
            "Secret",
            cls._static_request(
                "get",
                "/v1/apps/secrets/find",
                params=params,
            ),
        )

    @classmethod
    async def find_async(
        cls, **params: Unpack["Secret.FindParams"]
    ) -> "Secret":
        """
        Finds a secret in the secret store by name and scope.
        """
        return cast(
            "Secret",
            await cls._static_request_async(
                "get",
                "/v1/apps/secrets/find",
                params=params,
            ),
        )

    @classmethod
    def list(
        cls, **params: Unpack["Secret.ListParams"]
    ) -> ListObject["Secret"]:
        """
        List all secrets stored on the given scope.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):

            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["Secret.ListParams"]
    ) -> ListObject["Secret"]:
        """
        List all secrets stored on the given scope.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):

            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    _inner_class_types = {"scope": Scope}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_service import StripeService
from stripe.terminal._connection_token import ConnectionToken
from typing import List, cast
from typing_extensions import NotRequired, TypedDict


class ConnectionTokenService(StripeService):
    class CreateParams(TypedDict):
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """
        location: NotRequired["str"]
        """
        The id of the location that this connection token is scoped to. If specified the connection token will only be usable with readers assigned to that location, otherwise the connection token will be usable with all readers. Note that location scoping only applies to internet-connected readers. For more details, see [the docs on scoping connection tokens](https://stripe.com/docs/terminal/fleet/locations#connection-tokens).
        """

    def create(
        self,
        params: "ConnectionTokenService.CreateParams" = {},
        options: RequestOptions = {},
    ) -> ConnectionToken:
        """
        To connect to a reader the Stripe Terminal SDK needs to retrieve a short-lived connection token from Stripe, proxied through your server. On your backend, add an endpoint that creates and returns a connection token.
        """
        return cast(
            ConnectionToken,
            self._request(
                "post",
                "/v1/terminal/connection_tokens",
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "ConnectionTokenService.CreateParams" = {},
        options: RequestOptions = {},
    ) -> ConnectionToken:
        """
        To connect to a reader the Stripe Terminal SDK needs to retrieve a short-lived connection token from Stripe, proxied through your server. On your backend, add an endpoint that creates and returns a connection token.
        """
        return cast(
            ConnectionToken,
            await self._request_async(
                "post",
                "/v1/terminal/connection_tokens",
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe.radar._early_fraud_warning import (
    EarlyFraudWarning as EarlyFraudWarning,
)
from stripe.radar._early_fraud_warning_service import (
    EarlyFraudWarningService as EarlyFraudWarningService,
)
from stripe.radar._value_list import ValueList as ValueList
from stripe.radar._value_list_item import ValueListItem as ValueListItem
from stripe.radar._value_list_item_service import (
    ValueListItemService as ValueListItemService,
)
from stripe.radar._value_list_service import (
    ValueListService as ValueListService,
)



from io import BytesIO, IOBase
import json
import platform
from typing import (
    Any,
    AsyncIterable,
    Dict,
    List,
    Mapping,
    Optional,
    Tuple,
    Union,
    cast,
    ClassVar,
)
from typing_extensions import (
    TYPE_CHECKING,
    Literal,
    NoReturn,
    Unpack,
)
import uuid
from urllib.parse import urlsplit, urlunsplit

# breaking circular dependency
import stripe  # noqa: IMP101
from stripe._util import (
    log_debug,
    log_info,
    dashboard_link,
    _convert_to_stripe_object,
)
from stripe._version import VERSION
import stripe._error as error
import stripe.oauth_error as oauth_error
from stripe._multipart_data_generator import MultipartDataGenerator
from urllib.parse import urlencode
from stripe._encode import (
    _api_encode,
    _json_encode_date_callback,
)
from stripe._stripe_response import (
    StripeResponse,
    StripeStreamResponse,
    StripeStreamResponseAsync,
)
from stripe._request_options import RequestOptions, merge_options
from stripe._requestor_options import (
    RequestorOptions,
    _GlobalRequestorOptions,
)
from stripe._http_client import (
    HTTPClient,
    new_default_http_client,
    new_http_client_async_fallback,
)
from stripe._app_info import AppInfo

from stripe._base_address import BaseAddress
from stripe._api_mode import ApiMode

if TYPE_CHECKING:
    from stripe._stripe_object import StripeObject

HttpVerb = Literal["get", "post", "delete"]

# Lazily initialized
_default_proxy: Optional[str] = None


class _APIRequestor(object):
    _instance: ClassVar["_APIRequestor|None"] = None

    def __init__(
        self,
        options: RequestorOptions = RequestorOptions(),
        client: Optional[HTTPClient] = None,
    ):
        self._options = options
        self._client = client

    # In the case of client=None, we should use the current value of stripe.default_http_client
    # or lazily initialize it. Since stripe.default_http_client can change throughout the lifetime of
    # an _APIRequestor, we shouldn't set it as stripe._client and should access it only through this
    # getter.
    def _get_http_client(self) -> HTTPClient:
        client = self._client
        if client is None:
            global _default_proxy

            if not stripe.default_http_client:
                kwargs = {
                    "verify_ssl_certs": stripe.verify_ssl_certs,
                    "proxy": stripe.proxy,
                }
                # If the stripe.default_http_client has not been set by the user
                # yet, we'll set it here. This way, we aren't creating a new
                # HttpClient for every request.
                stripe.default_http_client = new_default_http_client(
                    async_fallback_client=new_http_client_async_fallback(
                        **kwargs
                    ),
                    **kwargs,
                )
                _default_proxy = stripe.proxy
            elif stripe.proxy != _default_proxy:
                import warnings

                warnings.warn(
                    "stripe.proxy was updated after sending a "
                    "request - this is a no-op. To use a different proxy, "
                    "set stripe.default_http_client to a new client "
                    "configured with the proxy."
                )

            assert stripe.default_http_client is not None
            return stripe.default_http_client
        return client

    def _replace_options(
        self, options: Optional[RequestOptions]
    ) -> "_APIRequestor":
        options = options or {}
        new_options = self._options.to_dict()
        for key in ["api_key", "stripe_account", "stripe_version"]:
            if key in options and options[key] is not None:
                new_options[key] = options[key]
        return _APIRequestor(
            options=RequestorOptions(**new_options), client=self._client
        )

    @property
    def api_key(self):
        return self._options.api_key

    @property
    def stripe_account(self):
        return self._options.stripe_account

    @property
    def stripe_version(self):
        return self._options.stripe_version

    @property
    def base_addresses(self):
        return self._options.base_addresses

    @classmethod
    def _global_instance(cls):
        """
        Returns the singleton instance of _APIRequestor, to be used when
        calling a static method such as stripe.Customer.create(...)
        """

        # Lazily initialize.
        if cls._instance is None:
            cls._instance = cls(options=_GlobalRequestorOptions(), client=None)
        return cls._instance

    @staticmethod
    def _global_with_options(
        **params: Unpack[RequestOptions],
    ) -> "_APIRequestor":
        return _APIRequestor._global_instance()._replace_options(params)

    @classmethod
    def _format_app_info(cls, info):
        str = info["name"]
        if info["version"]:
            str += "/%s" % (info["version"],)
        if info["url"]:
            str += " (%s)" % (info["url"],)
        return str

    def request(
        self,
        method: str,
        url: str,
        params: Optional[Mapping[str, Any]] = None,
        options: Optional[RequestOptions] = None,
        *,
        base_address: BaseAddress,
        api_mode: ApiMode,
        _usage: Optional[List[str]] = None,
    ) -> "StripeObject":
        requestor = self._replace_options(options)
        rbody, rcode, rheaders = requestor.request_raw(
            method.lower(),
            url,
            params,
            is_streaming=False,
            api_mode=api_mode,
            base_address=base_address,
            options=options,
            _usage=_usage,
        )
        resp = requestor._interpret_response(rbody, rcode, rheaders)

        return _convert_to_stripe_object(
            resp=resp,
            params=params,
            requestor=requestor,
            api_mode=api_mode,
        )

    async def request_async(
        self,
        method: str,
        url: str,
        params: Optional[Mapping[str, Any]] = None,
        options: Optional[RequestOptions] = None,
        *,
        base_address: BaseAddress,
        api_mode: ApiMode,
        _usage: Optional[List[str]] = None,
    ) -> "StripeObject":
        requestor = self._replace_options(options)
        rbody, rcode, rheaders = await requestor.request_raw_async(
            method.lower(),
            url,
            params,
            is_streaming=False,
            api_mode=api_mode,
            base_address=base_address,
            options=options,
            _usage=_usage,
        )
        resp = requestor._interpret_response(rbody, rcode, rheaders)

        return _convert_to_stripe_object(
            resp=resp,
            params=params,
            requestor=requestor,
            api_mode=api_mode,
        )

    def request_stream(
        self,
        method: str,
        url: str,
        params: Optional[Mapping[str, Any]] = None,
        options: Optional[RequestOptions] = None,
        *,
        base_address: BaseAddress,
        api_mode: ApiMode,
        _usage: Optional[List[str]] = None,
    ) -> StripeStreamResponse:
        stream, rcode, rheaders = self.request_raw(
            method.lower(),
            url,
            params,
            is_streaming=True,
            api_mode=api_mode,
            base_address=base_address,
            options=options,
            _usage=_usage,
        )
        resp = self._interpret_streaming_response(
            # TODO: should be able to remove this cast once self._client.request_stream_with_retries
            # returns a more specific type.
            cast(IOBase, stream),
            rcode,
            rheaders,
        )
        return resp

    async def request_stream_async(
        self,
        method: str,
        url: str,
        params: Optional[Mapping[str, Any]] = None,
        options: Optional[RequestOptions] = None,
        *,
        base_address: BaseAddress,
        api_mode: ApiMode,
        _usage: Optional[List[str]] = None,
    ) -> StripeStreamResponseAsync:
        stream, rcode, rheaders = await self.request_raw_async(
            method.lower(),
            url,
            params,
            is_streaming=True,
            api_mode=api_mode,
            base_address=base_address,
            options=options,
            _usage=_usage,
        )
        resp = await self._interpret_streaming_response_async(
            stream,
            rcode,
            rheaders,
        )
        return resp

    def handle_error_response(self, rbody, rcode, resp, rheaders) -> NoReturn:
        try:
            error_data = resp["error"]
        except (KeyError, TypeError):
            raise error.APIError(
                "Invalid response object from API: %r (HTTP response code "
                "was %d)" % (rbody, rcode),
                rbody,
                rcode,
                resp,
            )

        err = None

        # OAuth errors are a JSON object where `error` is a string. In
        # contrast, in API errors, `error` is a hash with sub-keys. We use
        # this property to distinguish between OAuth and API errors.
        if isinstance(error_data, str):
            err = self.specific_oauth_error(
                rbody, rcode, resp, rheaders, error_data
            )

        if err is None:
            err = self.specific_api_error(
                rbody, rcode, resp, rheaders, error_data
            )

        raise err

    def specific_api_error(self, rbody, rcode, resp, rheaders, error_data):
        log_info(
            "Stripe API error received",
            error_code=error_data.get("code"),
            error_type=error_data.get("type"),
            error_message=error_data.get("message"),
            error_param=error_data.get("param"),
        )

        # Rate limits were previously coded as 400's with code 'rate_limit'
        if rcode == 429 or (
            rcode == 400 and error_data.get("code") == "rate_limit"
        ):
            return error.RateLimitError(
                error_data.get("message"), rbody, rcode, resp, rheaders
            )
        elif rcode in [400, 404]:
            if error_data.get("type") == "idempotency_error":
                return error.IdempotencyError(
                    error_data.get("message"), rbody, rcode, resp, rheaders
                )
            else:
                return error.InvalidRequestError(
                    error_data.get("message"),
                    error_data.get("param"),
                    error_data.get("code"),
                    rbody,
                    rcode,
                    resp,
                    rheaders,
                )
        elif rcode == 401:
            return error.AuthenticationError(
                error_data.get("message"), rbody, rcode, resp, rheaders
            )
        elif rcode == 402:
            return error.CardError(
                error_data.get("message"),
                error_data.get("param"),
                error_data.get("code"),
                rbody,
                rcode,
                resp,
                rheaders,
            )
        elif rcode == 403:
            return error.PermissionError(
                error_data.get("message"), rbody, rcode, resp, rheaders
            )
        else:
            return error.APIError(
                error_data.get("message"), rbody, rcode, resp, rheaders
            )

    def specific_oauth_error(self, rbody, rcode, resp, rheaders, error_code):
        description = resp.get("error_description", error_code)

        log_info(
            "Stripe OAuth error received",
            error_code=error_code,
            error_description=description,
        )

        args = [error_code, description, rbody, rcode, resp, rheaders]

        if error_code == "invalid_client":
            return oauth_error.InvalidClientError(*args)
        elif error_code == "invalid_grant":
            return oauth_error.InvalidGrantError(*args)
        elif error_code == "invalid_request":
            return oauth_error.InvalidRequestError(*args)
        elif error_code == "invalid_scope":
            return oauth_error.InvalidScopeError(*args)
        elif error_code == "unsupported_grant_type":
            return oauth_error.UnsupportedGrantTypeError(*args)
        elif error_code == "unsupported_response_type":
            return oauth_error.UnsupportedResponseTypeError(*args)

        return None

    def request_headers(
        self, method, options: RequestOptions, api_mode: ApiMode
    ):
        user_agent = "Stripe/v1 PythonBindings/%s" % (VERSION,)
        if stripe.app_info:
            user_agent += " " + self._format_app_info(stripe.app_info)

        ua: Dict[str, Union[str, AppInfo]] = {
            "bindings_version": VERSION,
            "lang": "python",
            "publisher": "stripe",
            "httplib": self._get_http_client().name,
        }
        for attr, func in [
            ["lang_version", platform.python_version],
            ["platform", platform.platform],
            ["uname", lambda: " ".join(platform.uname())],
        ]:
            try:
                val = func()
            except Exception:
                val = "(disabled)"
            ua[attr] = val
        if stripe.app_info:
            ua["application"] = stripe.app_info

        headers: Dict[str, str] = {
            "X-Stripe-Client-User-Agent": json.dumps(ua),
            "User-Agent": user_agent,
            "Authorization": "Bearer %s" % (options.get("api_key"),),
        }

        stripe_account = options.get("stripe_account")
        if stripe_account:
            headers["Stripe-Account"] = stripe_account

        idempotency_key = options.get("idempotency_key")
        if idempotency_key:
            headers["Idempotency-Key"] = idempotency_key

        if method == "post":
            headers.setdefault("Idempotency-Key", str(uuid.uuid4()))
            if api_mode == "preview":
                headers["Content-Type"] = "application/json"
            else:
                headers["Content-Type"] = "application/x-www-form-urlencoded"

        stripe_version = options.get("stripe_version")
        if stripe_version:
            headers["Stripe-Version"] = stripe_version

        return headers

    def _args_for_request_with_retries(
        self,
        method: str,
        url: str,
        params: Optional[Mapping[str, Any]] = None,
        options: Optional[RequestOptions] = None,
        *,
        base_address: BaseAddress,
        api_mode: ApiMode,
        _usage: Optional[List[str]] = None,
    ):
        """
        Mechanism for issuing an API call
        """
        request_options = merge_options(self._options, options)

        # Special stripe_version handling for preview requests:
        if (
            options
            and "stripe_version" in options
            and (options["stripe_version"] is not None)
        ):
            # If user specified an API version, honor it
            request_options["stripe_version"] = options["stripe_version"]
        elif api_mode == "preview":
            request_options["stripe_version"] = stripe.preview_api_version

        if request_options.get("api_key") is None:
            raise error.AuthenticationError(
                "No API key provided. (HINT: set your API key using "
                '"stripe.api_key = <API-KEY>"). You can generate API keys '
                "from the Stripe web interface.  See https://stripe.com/api "
                "for details, or email support@stripe.com if you have any "
                "questions."
            )

        abs_url = "%s%s" % (
            self._options.base_addresses.get(base_address),
            url,
        )

        encoded_params = urlencode(list(_api_encode(params or {})))

        # Don't use strict form encoding by changing the square bracket control
        # characters back to their literals. This is fine by the server, and
        # makes these parameter strings easier to read.
        encoded_params = encoded_params.replace("%5B", "[").replace("%5D", "]")

        if api_mode == "preview":
            encoded_body = json.dumps(
                params or {}, default=_json_encode_date_callback
            )
        else:
            encoded_body = encoded_params

        supplied_headers = None
        if (
            "headers" in request_options
            and request_options["headers"] is not None
        ):
            supplied_headers = dict(request_options["headers"])

        headers = self.request_headers(method, request_options, api_mode)

        if method == "get" or method == "delete":
            if params:
                query = encoded_params
                scheme, netloc, path, base_query, fragment = urlsplit(abs_url)

                if base_query:
                    query = "%s&%s" % (base_query, query)

                abs_url = urlunsplit((scheme, netloc, path, query, fragment))
            post_data = None
        elif method == "post":
            if api_mode == "V1FILES":
                generator = MultipartDataGenerator()
                generator.add_params(params or {})
                post_data = generator.get_post_data()
                headers[
                    "Content-Type"
                ] = "multipart/form-data; boundary=%s" % (generator.boundary,)
            else:
                post_data = encoded_body
        else:
            raise error.APIConnectionError(
                "Unrecognized HTTP method %r.  This may indicate a bug in the "
                "Stripe bindings.  Please contact support@stripe.com for "
                "assistance." % (method,)
            )

        if supplied_headers is not None:
            for key, value in supplied_headers.items():
                headers[key] = value

        max_network_retries = request_options.get("max_network_retries")

        return (
            # Actual args
            method,
            abs_url,
            headers,
            post_data,
            max_network_retries,
            _usage,
            # For logging
            encoded_params,
            request_options.get("stripe_version"),
        )

    def request_raw(
        self,
        method: str,
        url: str,
        params: Optional[Mapping[str, Any]] = None,
        options: Optional[RequestOptions] = None,
        is_streaming: bool = False,
        *,
        base_address: BaseAddress,
        api_mode: ApiMode,
        _usage: Optional[List[str]] = None,
    ) -> Tuple[object, int, Mapping[str, str]]:
        (
            method,
            abs_url,
            headers,
            post_data,
            max_network_retries,
            _usage,
            encoded_params,
            api_version,
        ) = self._args_for_request_with_retries(
            method,
            url,
            params,
            options,
            base_address=base_address,
            api_mode=api_mode,
            _usage=_usage,
        )

        log_info("Request to Stripe api", method=method, url=abs_url)
        log_debug(
            "Post details", post_data=encoded_params, api_version=api_version
        )

        if is_streaming:
            (
                rcontent,
                rcode,
                rheaders,
            ) = self._get_http_client().request_stream_with_retries(
                method,
                abs_url,
                headers,
                post_data,
                max_network_retries=max_network_retries,
                _usage=_usage,
            )
        else:
            (
                rcontent,
                rcode,
                rheaders,
            ) = self._get_http_client().request_with_retries(
                method,
                abs_url,
                headers,
                post_data,
                max_network_retries=max_network_retries,
                _usage=_usage,
            )

        log_info("Stripe API response", path=abs_url, response_code=rcode)
        log_debug("API response body", body=rcontent)

        if "Request-Id" in rheaders:
            request_id = rheaders["Request-Id"]
            log_debug(
                "Dashboard link for request",
                link=dashboard_link(request_id),
            )

        return rcontent, rcode, rheaders

    async def request_raw_async(
        self,
        method: str,
        url: str,
        params: Optional[Mapping[str, Any]] = None,
        options: Optional[RequestOptions] = None,
        is_streaming: bool = False,
        *,
        base_address: BaseAddress,
        api_mode: ApiMode,
        _usage: Optional[List[str]] = None,
    ) -> Tuple[AsyncIterable[bytes], int, Mapping[str, str]]:
        """
        Mechanism for issuing an API call
        """

        # TODO - we can DRY this up, but we should do it in master too to avoid a perpetual source
        # of merge conflicts.
        request_options = merge_options(self._options, options)

        # Special stripe_version handling for preview requests:
        if (
            options
            and "stripe_version" in options
            and (options["stripe_version"] is not None)
        ):
            # If user specified an API version, honor it
            request_options["stripe_version"] = options["stripe_version"]
        elif api_mode == "preview":
            request_options["stripe_version"] = stripe.preview_api_version

        if request_options.get("api_key") is None:
            raise error.AuthenticationError(
                "No API key provided. (HINT: set your API key using "
                '"stripe.api_key = <API-KEY>"). You can generate API keys '
                "from the Stripe web interface.  See https://stripe.com/api "
                "for details, or email support@stripe.com if you have any "
                "questions."
            )

        abs_url = "%s%s" % (
            self._options.base_addresses.get(base_address),
            url,
        )

        encoded_params = urlencode(list(_api_encode(params or {})))

        # Don't use strict form encoding by changing the square bracket control
        # characters back to their literals. This is fine by the server, and
        # makes these parameter strings easier to read.
        encoded_params = encoded_params.replace("%5B", "[").replace("%5D", "]")

        if api_mode == "preview":
            encoded_body = json.dumps(
                params or {}, default=_json_encode_date_callback
            )
        else:
            encoded_body = encoded_params

        supplied_headers = None
        if (
            "headers" in request_options
            and request_options["headers"] is not None
        ):
            supplied_headers = dict(request_options["headers"])

        headers = self.request_headers(method, request_options, api_mode)

        if method == "get" or method == "delete":
            if params:
                query = encoded_params
                scheme, netloc, path, base_query, fragment = urlsplit(abs_url)

                if base_query:
                    query = "%s&%s" % (base_query, query)

                abs_url = urlunsplit((scheme, netloc, path, query, fragment))
            post_data = None
        elif method == "post":
            if api_mode == "V1FILES":
                generator = MultipartDataGenerator()
                generator.add_params(params or {})
                post_data = generator.get_post_data()
                headers[
                    "Content-Type"
                ] = "multipart/form-data; boundary=%s" % (generator.boundary,)
            else:
                post_data = encoded_body
        else:
            raise error.APIConnectionError(
                "Unrecognized HTTP method %r.  This may indicate a bug in the "
                "Stripe bindings.  Please contact support@stripe.com for "
                "assistance." % (method,)
            )

        if supplied_headers is not None:
            for key, value in supplied_headers.items():
                headers[key] = value

        log_info("Request to Stripe api", method=method, url=abs_url)
        log_debug(
            "Post details",
            post_data=encoded_params,
            api_version=request_options.get("stripe_version"),
        )

        if is_streaming:
            (
                rcontent,
                rcode,
                rheaders,
            ) = await self._get_http_client().request_stream_with_retries_async(
                method,
                abs_url,
                headers,
                post_data,
                max_network_retries=request_options.get("max_network_retries"),
                _usage=_usage,
            )
        else:
            (
                rcontent,
                rcode,
                rheaders,
            ) = await self._get_http_client().request_with_retries_async(
                method,
                abs_url,
                headers,
                post_data,
                max_network_retries=request_options.get("max_network_retries"),
                _usage=_usage,
            )

        log_info("Stripe API response", path=abs_url, response_code=rcode)
        log_debug("API response body", body=rcontent)

        if "Request-Id" in rheaders:
            request_id = rheaders["Request-Id"]
            log_debug(
                "Dashboard link for request",
                link=dashboard_link(request_id),
            )

        return rcontent, rcode, rheaders

    def _should_handle_code_as_error(self, rcode):
        return not 200 <= rcode < 300

    def _interpret_response(
        self,
        rbody: object,
        rcode: int,
        rheaders: Mapping[str, str],
    ) -> StripeResponse:
        try:
            if hasattr(rbody, "decode"):
                # TODO: should be able to remove this cast once self._client.request_with_retries
                # returns a more specific type.
                rbody = cast(bytes, rbody).decode("utf-8")
            resp = StripeResponse(
                cast(str, rbody),
                rcode,
                rheaders,
            )
        except Exception:
            raise error.APIError(
                "Invalid response body from API: %s "
                "(HTTP response code was %d)" % (rbody, rcode),
                cast(bytes, rbody),
                rcode,
                rheaders,
            )
        if self._should_handle_code_as_error(rcode):
            self.handle_error_response(rbody, rcode, resp.data, rheaders)
        return resp

    async def _interpret_streaming_response_async(
        self,
        stream: AsyncIterable[bytes],
        rcode: int,
        rheaders: Mapping[str, str],
    ) -> StripeStreamResponseAsync:
        if self._should_handle_code_as_error(rcode):
            json_content = b"".join([chunk async for chunk in stream])
            self._interpret_response(json_content, rcode, rheaders)
            # _interpret_response is guaranteed to throw since we've checked self._should_handle_code_as_error
            raise RuntimeError(
                "_interpret_response should have raised an error"
            )
        else:
            return StripeStreamResponseAsync(stream, rcode, rheaders)

    def _interpret_streaming_response(
        self,
        stream: IOBase,
        rcode: int,
        rheaders: Mapping[str, str],
    ) -> StripeStreamResponse:
        # Streaming response are handled with minimal processing for the success
        # case (ie. we don't want to read the content). When an error is
        # received, we need to read from the stream and parse the received JSON,
        # treating it like a standard JSON response.
        if self._should_handle_code_as_error(rcode):
            if hasattr(stream, "getvalue"):
                json_content = cast(BytesIO, stream).getvalue()
            elif hasattr(stream, "read"):
                json_content = stream.read()
            else:
                raise NotImplementedError(
                    "HTTP client %s does not return an IOBase object which "
                    "can be consumed when streaming a response."
                    % self._get_http_client().name
                )

            self._interpret_response(json_content, rcode, rheaders)
            # _interpret_response is guaranteed to throw since we've checked self._should_handle_code_as_error
            raise RuntimeError(
                "_interpret_response should have raised an error"
            )
        else:
            return StripeStreamResponse(stream, rcode, rheaders)



# -*- coding: utf-8 -*-
from typing_extensions import TYPE_CHECKING
from warnings import warn

warn(
    """
    The stripe.api_resources.error_object package is deprecated, please change your
    imports to import from stripe directly.
    From:
      from stripe.api_resources.error_object import ErrorObject
    To:
      from stripe import ErrorObject
    """,
    DeprecationWarning,
)

if not TYPE_CHECKING:
    from stripe._error import StripeError  # noqa
    from stripe._error import APIError  # noqa
    from stripe._error import APIConnectionError  # noqa
    from stripe._error import StripeErrorWithParamCode  # noqa
    from stripe._error import CardError  # noqa
    from stripe._error import IdempotencyError  # noqa
    from stripe._error import InvalidRequestError  # noqa
    from stripe._error import AuthenticationError  # noqa
    from stripe._error import PermissionError  # noqa
    from stripe._error import RateLimitError  # noqa
    from stripe._error import SignatureVerificationError  # noqa



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._account_notice import AccountNotice
from stripe._list_object import ListObject
from stripe._request_options import RequestOptions
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Dict, List, cast
from typing_extensions import NotRequired, TypedDict


class AccountNoticeService(StripeService):
    class ListParams(TypedDict):
        ending_before: NotRequired["str"]
        """
        A cursor for use in pagination. `ending_before` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, starting with `obj_bar`, your subsequent call can include `ending_before=obj_bar` in order to fetch the previous page of the list.
        """
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """
        limit: NotRequired["int"]
        """
        A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
        """
        sent: NotRequired["bool"]
        """
        Set to false to only return unsent AccountNotices.
        """
        starting_after: NotRequired["str"]
        """
        A cursor for use in pagination. `starting_after` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with `obj_foo`, your subsequent call can include `starting_after=obj_foo` in order to fetch the next page of the list.
        """

    class RetrieveParams(TypedDict):
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """

    class UpdateParams(TypedDict):
        email: "AccountNoticeService.UpdateParamsEmail"
        """
        Information about the email you sent.
        """
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """
        metadata: NotRequired["Dict[str, str]"]
        """
        Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
        """
        sent_at: int
        """
        Date when you sent the notice.
        """

    class UpdateParamsEmail(TypedDict):
        plain_text: str
        """
        Content of the email in plain text. The copy must match exactly the language that Stripe Compliance has approved for use.
        """
        recipient: str
        """
        Email address of the recipient.
        """
        subject: str
        """
        Subject of the email.
        """

    def list(
        self,
        params: "AccountNoticeService.ListParams" = {},
        options: RequestOptions = {},
    ) -> ListObject[AccountNotice]:
        """
        Retrieves a list of AccountNotice objects. The objects are sorted in descending order by creation date, with the most-recently-created object appearing first.
        """
        return cast(
            ListObject[AccountNotice],
            self._request(
                "get",
                "/v1/account_notices",
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: "AccountNoticeService.ListParams" = {},
        options: RequestOptions = {},
    ) -> ListObject[AccountNotice]:
        """
        Retrieves a list of AccountNotice objects. The objects are sorted in descending order by creation date, with the most-recently-created object appearing first.
        """
        return cast(
            ListObject[AccountNotice],
            await self._request_async(
                "get",
                "/v1/account_notices",
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        account_notice: str,
        params: "AccountNoticeService.RetrieveParams" = {},
        options: RequestOptions = {},
    ) -> AccountNotice:
        """
        Retrieves an AccountNotice object.
        """
        return cast(
            AccountNotice,
            self._request(
                "get",
                "/v1/account_notices/{account_notice}".format(
                    account_notice=sanitize_id(account_notice),
                ),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        account_notice: str,
        params: "AccountNoticeService.RetrieveParams" = {},
        options: RequestOptions = {},
    ) -> AccountNotice:
        """
        Retrieves an AccountNotice object.
        """
        return cast(
            AccountNotice,
            await self._request_async(
                "get",
                "/v1/account_notices/{account_notice}".format(
                    account_notice=sanitize_id(account_notice),
                ),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        account_notice: str,
        params: "AccountNoticeService.UpdateParams",
        options: RequestOptions = {},
    ) -> AccountNotice:
        """
        Updates an AccountNotice object.
        """
        return cast(
            AccountNotice,
            self._request(
                "post",
                "/v1/account_notices/{account_notice}".format(
                    account_notice=sanitize_id(account_notice),
                ),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        account_notice: str,
        params: "AccountNoticeService.UpdateParams",
        options: RequestOptions = {},
    ) -> AccountNotice:
        """
        Updates an AccountNotice object.
        """
        return cast(
            AccountNotice,
            await self._request_async(
                "post",
                "/v1/account_notices/{account_notice}".format(
                    account_notice=sanitize_id(account_notice),
                ),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._list_object import ListObject
from stripe._person import Person
from stripe._request_options import RequestOptions
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Dict, List, cast
from typing_extensions import Literal, NotRequired, TypedDict


class AccountPersonService(StripeService):
    class CreateParams(TypedDict):
        additional_tos_acceptances: NotRequired[
            "AccountPersonService.CreateParamsAdditionalTosAcceptances"
        ]
        """
        Details on the legal guardian's acceptance of the required Stripe agreements.
        """
        address: NotRequired["AccountPersonService.CreateParamsAddress"]
        """
        The person's address.
        """
        address_kana: NotRequired[
            "AccountPersonService.CreateParamsAddressKana"
        ]
        """
        The Kana variation of the person's address (Japan only).
        """
        address_kanji: NotRequired[
            "AccountPersonService.CreateParamsAddressKanji"
        ]
        """
        The Kanji variation of the person's address (Japan only).
        """
        dob: NotRequired["Literal['']|AccountPersonService.CreateParamsDob"]
        """
        The person's date of birth.
        """
        documents: NotRequired["AccountPersonService.CreateParamsDocuments"]
        """
        Documents that may be submitted to satisfy various informational requests.
        """
        email: NotRequired["str"]
        """
        The person's email address.
        """
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """
        first_name: NotRequired["str"]
        """
        The person's first name.
        """
        first_name_kana: NotRequired["str"]
        """
        The Kana variation of the person's first name (Japan only).
        """
        first_name_kanji: NotRequired["str"]
        """
        The Kanji variation of the person's first name (Japan only).
        """
        full_name_aliases: NotRequired["Literal['']|List[str]"]
        """
        A list of alternate names or aliases that the person is known by.
        """
        gender: NotRequired["str"]
        """
        The person's gender (International regulations require either "male" or "female").
        """
        id_number: NotRequired["str"]
        """
        The person's ID number, as appropriate for their country. For example, a social security number in the U.S., social insurance number in Canada, etc. Instead of the number itself, you can also provide a [PII token provided by Stripe.js](https://stripe.com/docs/js/tokens/create_token?type=pii).
        """
        id_number_secondary: NotRequired["str"]
        """
        The person's secondary ID number, as appropriate for their country, will be used for enhanced verification checks. In Thailand, this would be the laser code found on the back of an ID card. Instead of the number itself, you can also provide a [PII token provided by Stripe.js](https://stripe.com/docs/js/tokens/create_token?type=pii).
        """
        last_name: NotRequired["str"]
        """
        The person's last name.
        """
        last_name_kana: NotRequired["str"]
        """
        The Kana variation of the person's last name (Japan only).
        """
        last_name_kanji: NotRequired["str"]
        """
        The Kanji variation of the person's last name (Japan only).
        """
        maiden_name: NotRequired["str"]
        """
        The person's maiden name.
        """
        metadata: NotRequired["Literal['']|Dict[str, str]"]
        """
        Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
        """
        nationality: NotRequired["str"]
        """
        The country where the person is a national. Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)), or "XX" if unavailable.
        """
        person_token: NotRequired["str"]
        """
        A [person token](https://stripe.com/docs/connect/account-tokens), used to securely provide details to the person.
        """
        phone: NotRequired["str"]
        """
        The person's phone number.
        """
        political_exposure: NotRequired["str"]
        """
        Indicates if the person or any of their representatives, family members, or other closely related persons, declares that they hold or have held an important public job or function, in any jurisdiction.
        """
        registered_address: NotRequired[
            "AccountPersonService.CreateParamsRegisteredAddress"
        ]
        """
        The person's registered address.
        """
        relationship: NotRequired[
            "AccountPersonService.CreateParamsRelationship"
        ]
        """
        The relationship that this person has with the account's legal entity.
        """
        ssn_last_4: NotRequired["str"]
        """
        The last four digits of the person's Social Security number (U.S. only).
        """
        verification: NotRequired[
            "AccountPersonService.CreateParamsVerification"
        ]
        """
        The person's verification status.
        """

    class CreateParamsAdditionalTosAcceptances(TypedDict):
        account: NotRequired[
            "AccountPersonService.CreateParamsAdditionalTosAcceptancesAccount"
        ]
        """
        Details on the legal guardian's acceptance of the main Stripe service agreement.
        """

    class CreateParamsAdditionalTosAcceptancesAccount(TypedDict):
        date: NotRequired["int"]
        """
        The Unix timestamp marking when the account representative accepted the service agreement.
        """
        ip: NotRequired["str"]
        """
        The IP address from which the account representative accepted the service agreement.
        """
        user_agent: NotRequired["Literal['']|str"]
        """
        The user agent of the browser from which the account representative accepted the service agreement.
        """

    class CreateParamsAddress(TypedDict):
        city: NotRequired["str"]
        """
        City, district, suburb, town, or village.
        """
        country: NotRequired["str"]
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: NotRequired["str"]
        """
        Address line 1 (e.g., street, PO Box, or company name).
        """
        line2: NotRequired["str"]
        """
        Address line 2 (e.g., apartment, suite, unit, or building).
        """
        postal_code: NotRequired["str"]
        """
        ZIP or postal code.
        """
        state: NotRequired["str"]
        """
        State, county, province, or region.
        """

    class CreateParamsAddressKana(TypedDict):
        city: NotRequired["str"]
        """
        City or ward.
        """
        country: NotRequired["str"]
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: NotRequired["str"]
        """
        Block or building number.
        """
        line2: NotRequired["str"]
        """
        Building details.
        """
        postal_code: NotRequired["str"]
        """
        Postal code.
        """
        state: NotRequired["str"]
        """
        Prefecture.
        """
        town: NotRequired["str"]
        """
        Town or cho-me.
        """

    class CreateParamsAddressKanji(TypedDict):
        city: NotRequired["str"]
        """
        City or ward.
        """
        country: NotRequired["str"]
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: NotRequired["str"]
        """
        Block or building number.
        """
        line2: NotRequired["str"]
        """
        Building details.
        """
        postal_code: NotRequired["str"]
        """
        Postal code.
        """
        state: NotRequired["str"]
        """
        Prefecture.
        """
        town: NotRequired["str"]
        """
        Town or cho-me.
        """

    class CreateParamsDob(TypedDict):
        day: int
        """
        The day of birth, between 1 and 31.
        """
        month: int
        """
        The month of birth, between 1 and 12.
        """
        year: int
        """
        The four-digit year of birth.
        """

    class CreateParamsDocuments(TypedDict):
        company_authorization: NotRequired[
            "AccountPersonService.CreateParamsDocumentsCompanyAuthorization"
        ]
        """
        One or more documents that demonstrate proof that this person is authorized to represent the company.
        """
        passport: NotRequired[
            "AccountPersonService.CreateParamsDocumentsPassport"
        ]
        """
        One or more documents showing the person's passport page with photo and personal data.
        """
        visa: NotRequired["AccountPersonService.CreateParamsDocumentsVisa"]
        """
        One or more documents showing the person's visa required for living in the country where they are residing.
        """

    class CreateParamsDocumentsCompanyAuthorization(TypedDict):
        files: NotRequired["List[str]"]
        """
        One or more document ids returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `account_requirement`.
        """

    class CreateParamsDocumentsPassport(TypedDict):
        files: NotRequired["List[str]"]
        """
        One or more document ids returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `account_requirement`.
        """

    class CreateParamsDocumentsVisa(TypedDict):
        files: NotRequired["List[str]"]
        """
        One or more document ids returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `account_requirement`.
        """

    class CreateParamsRegisteredAddress(TypedDict):
        city: NotRequired["str"]
        """
        City, district, suburb, town, or village.
        """
        country: NotRequired["str"]
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: NotRequired["str"]
        """
        Address line 1 (e.g., street, PO Box, or company name).
        """
        line2: NotRequired["str"]
        """
        Address line 2 (e.g., apartment, suite, unit, or building).
        """
        postal_code: NotRequired["str"]
        """
        ZIP or postal code.
        """
        state: NotRequired["str"]
        """
        State, county, province, or region.
        """

    class CreateParamsRelationship(TypedDict):
        director: NotRequired["bool"]
        """
        Whether the person is a director of the account's legal entity. Directors are typically members of the governing board of the company, or responsible for ensuring the company meets its regulatory obligations.
        """
        executive: NotRequired["bool"]
        """
        Whether the person has significant responsibility to control, manage, or direct the organization.
        """
        legal_guardian: NotRequired["bool"]
        """
        Whether the person is the legal guardian of the account's representative.
        """
        owner: NotRequired["bool"]
        """
        Whether the person is an owner of the account's legal entity.
        """
        percent_ownership: NotRequired["Literal['']|float"]
        """
        The percent owned by the person of the account's legal entity.
        """
        representative: NotRequired["bool"]
        """
        Whether the person is authorized as the primary representative of the account. This is the person nominated by the business to provide information about themselves, and general information about the account. There can only be one representative at any given time. At the time the account is created, this person should be set to the person responsible for opening the account.
        """
        title: NotRequired["str"]
        """
        The person's title (e.g., CEO, Support Engineer).
        """

    class CreateParamsVerification(TypedDict):
        additional_document: NotRequired[
            "AccountPersonService.CreateParamsVerificationAdditionalDocument"
        ]
        """
        A document showing address, either a passport, local ID card, or utility bill from a well-known utility company.
        """
        document: NotRequired[
            "AccountPersonService.CreateParamsVerificationDocument"
        ]
        """
        An identifying document, either a passport or local ID card.
        """

    class CreateParamsVerificationAdditionalDocument(TypedDict):
        back: NotRequired["str"]
        """
        The back of an ID returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `identity_document`. The uploaded file needs to be a color image (smaller than 8,000px by 8,000px), in JPG, PNG, or PDF format, and less than 10 MB in size.
        """
        front: NotRequired["str"]
        """
        The front of an ID returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `identity_document`. The uploaded file needs to be a color image (smaller than 8,000px by 8,000px), in JPG, PNG, or PDF format, and less than 10 MB in size.
        """

    class CreateParamsVerificationDocument(TypedDict):
        back: NotRequired["str"]
        """
        The back of an ID returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `identity_document`. The uploaded file needs to be a color image (smaller than 8,000px by 8,000px), in JPG, PNG, or PDF format, and less than 10 MB in size.
        """
        front: NotRequired["str"]
        """
        The front of an ID returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `identity_document`. The uploaded file needs to be a color image (smaller than 8,000px by 8,000px), in JPG, PNG, or PDF format, and less than 10 MB in size.
        """

    class DeleteParams(TypedDict):
        pass

    class ListParams(TypedDict):
        ending_before: NotRequired["str"]
        """
        A cursor for use in pagination. `ending_before` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, starting with `obj_bar`, your subsequent call can include `ending_before=obj_bar` in order to fetch the previous page of the list.
        """
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """
        limit: NotRequired["int"]
        """
        A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
        """
        relationship: NotRequired[
            "AccountPersonService.ListParamsRelationship"
        ]
        """
        Filters on the list of people returned based on the person's relationship to the account's company.
        """
        starting_after: NotRequired["str"]
        """
        A cursor for use in pagination. `starting_after` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with `obj_foo`, your subsequent call can include `starting_after=obj_foo` in order to fetch the next page of the list.
        """

    class ListParamsRelationship(TypedDict):
        director: NotRequired["bool"]
        """
        A filter on the list of people returned based on whether these people are directors of the account's company.
        """
        executive: NotRequired["bool"]
        """
        A filter on the list of people returned based on whether these people are executives of the account's company.
        """
        legal_guardian: NotRequired["bool"]
        """
        A filter on the list of people returned based on whether these people are legal guardians of the account's representative.
        """
        owner: NotRequired["bool"]
        """
        A filter on the list of people returned based on whether these people are owners of the account's company.
        """
        representative: NotRequired["bool"]
        """
        A filter on the list of people returned based on whether these people are the representative of the account's company.
        """

    class RetrieveParams(TypedDict):
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """

    class UpdateParams(TypedDict):
        additional_tos_acceptances: NotRequired[
            "AccountPersonService.UpdateParamsAdditionalTosAcceptances"
        ]
        """
        Details on the legal guardian's acceptance of the required Stripe agreements.
        """
        address: NotRequired["AccountPersonService.UpdateParamsAddress"]
        """
        The person's address.
        """
        address_kana: NotRequired[
            "AccountPersonService.UpdateParamsAddressKana"
        ]
        """
        The Kana variation of the person's address (Japan only).
        """
        address_kanji: NotRequired[
            "AccountPersonService.UpdateParamsAddressKanji"
        ]
        """
        The Kanji variation of the person's address (Japan only).
        """
        dob: NotRequired["Literal['']|AccountPersonService.UpdateParamsDob"]
        """
        The person's date of birth.
        """
        documents: NotRequired["AccountPersonService.UpdateParamsDocuments"]
        """
        Documents that may be submitted to satisfy various informational requests.
        """
        email: NotRequired["str"]
        """
        The person's email address.
        """
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """
        first_name: NotRequired["str"]
        """
        The person's first name.
        """
        first_name_kana: NotRequired["str"]
        """
        The Kana variation of the person's first name (Japan only).
        """
        first_name_kanji: NotRequired["str"]
        """
        The Kanji variation of the person's first name (Japan only).
        """
        full_name_aliases: NotRequired["Literal['']|List[str]"]
        """
        A list of alternate names or aliases that the person is known by.
        """
        gender: NotRequired["str"]
        """
        The person's gender (International regulations require either "male" or "female").
        """
        id_number: NotRequired["str"]
        """
        The person's ID number, as appropriate for their country. For example, a social security number in the U.S., social insurance number in Canada, etc. Instead of the number itself, you can also provide a [PII token provided by Stripe.js](https://stripe.com/docs/js/tokens/create_token?type=pii).
        """
        id_number_secondary: NotRequired["str"]
        """
        The person's secondary ID number, as appropriate for their country, will be used for enhanced verification checks. In Thailand, this would be the laser code found on the back of an ID card. Instead of the number itself, you can also provide a [PII token provided by Stripe.js](https://stripe.com/docs/js/tokens/create_token?type=pii).
        """
        last_name: NotRequired["str"]
        """
        The person's last name.
        """
        last_name_kana: NotRequired["str"]
        """
        The Kana variation of the person's last name (Japan only).
        """
        last_name_kanji: NotRequired["str"]
        """
        The Kanji variation of the person's last name (Japan only).
        """
        maiden_name: NotRequired["str"]
        """
        The person's maiden name.
        """
        metadata: NotRequired["Literal['']|Dict[str, str]"]
        """
        Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
        """
        nationality: NotRequired["str"]
        """
        The country where the person is a national. Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)), or "XX" if unavailable.
        """
        person_token: NotRequired["str"]
        """
        A [person token](https://stripe.com/docs/connect/account-tokens), used to securely provide details to the person.
        """
        phone: NotRequired["str"]
        """
        The person's phone number.
        """
        political_exposure: NotRequired["str"]
        """
        Indicates if the person or any of their representatives, family members, or other closely related persons, declares that they hold or have held an important public job or function, in any jurisdiction.
        """
        registered_address: NotRequired[
            "AccountPersonService.UpdateParamsRegisteredAddress"
        ]
        """
        The person's registered address.
        """
        relationship: NotRequired[
            "AccountPersonService.UpdateParamsRelationship"
        ]
        """
        The relationship that this person has with the account's legal entity.
        """
        ssn_last_4: NotRequired["str"]
        """
        The last four digits of the person's Social Security number (U.S. only).
        """
        verification: NotRequired[
            "AccountPersonService.UpdateParamsVerification"
        ]
        """
        The person's verification status.
        """

    class UpdateParamsAdditionalTosAcceptances(TypedDict):
        account: NotRequired[
            "AccountPersonService.UpdateParamsAdditionalTosAcceptancesAccount"
        ]
        """
        Details on the legal guardian's acceptance of the main Stripe service agreement.
        """

    class UpdateParamsAdditionalTosAcceptancesAccount(TypedDict):
        date: NotRequired["int"]
        """
        The Unix timestamp marking when the account representative accepted the service agreement.
        """
        ip: NotRequired["str"]
        """
        The IP address from which the account representative accepted the service agreement.
        """
        user_agent: NotRequired["Literal['']|str"]
        """
        The user agent of the browser from which the account representative accepted the service agreement.
        """

    class UpdateParamsAddress(TypedDict):
        city: NotRequired["str"]
        """
        City, district, suburb, town, or village.
        """
        country: NotRequired["str"]
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: NotRequired["str"]
        """
        Address line 1 (e.g., street, PO Box, or company name).
        """
        line2: NotRequired["str"]
        """
        Address line 2 (e.g., apartment, suite, unit, or building).
        """
        postal_code: NotRequired["str"]
        """
        ZIP or postal code.
        """
        state: NotRequired["str"]
        """
        State, county, province, or region.
        """

    class UpdateParamsAddressKana(TypedDict):
        city: NotRequired["str"]
        """
        City or ward.
        """
        country: NotRequired["str"]
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: NotRequired["str"]
        """
        Block or building number.
        """
        line2: NotRequired["str"]
        """
        Building details.
        """
        postal_code: NotRequired["str"]
        """
        Postal code.
        """
        state: NotRequired["str"]
        """
        Prefecture.
        """
        town: NotRequired["str"]
        """
        Town or cho-me.
        """

    class UpdateParamsAddressKanji(TypedDict):
        city: NotRequired["str"]
        """
        City or ward.
        """
        country: NotRequired["str"]
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: NotRequired["str"]
        """
        Block or building number.
        """
        line2: NotRequired["str"]
        """
        Building details.
        """
        postal_code: NotRequired["str"]
        """
        Postal code.
        """
        state: NotRequired["str"]
        """
        Prefecture.
        """
        town: NotRequired["str"]
        """
        Town or cho-me.
        """

    class UpdateParamsDob(TypedDict):
        day: int
        """
        The day of birth, between 1 and 31.
        """
        month: int
        """
        The month of birth, between 1 and 12.
        """
        year: int
        """
        The four-digit year of birth.
        """

    class UpdateParamsDocuments(TypedDict):
        company_authorization: NotRequired[
            "AccountPersonService.UpdateParamsDocumentsCompanyAuthorization"
        ]
        """
        One or more documents that demonstrate proof that this person is authorized to represent the company.
        """
        passport: NotRequired[
            "AccountPersonService.UpdateParamsDocumentsPassport"
        ]
        """
        One or more documents showing the person's passport page with photo and personal data.
        """
        visa: NotRequired["AccountPersonService.UpdateParamsDocumentsVisa"]
        """
        One or more documents showing the person's visa required for living in the country where they are residing.
        """

    class UpdateParamsDocumentsCompanyAuthorization(TypedDict):
        files: NotRequired["List[str]"]
        """
        One or more document ids returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `account_requirement`.
        """

    class UpdateParamsDocumentsPassport(TypedDict):
        files: NotRequired["List[str]"]
        """
        One or more document ids returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `account_requirement`.
        """

    class UpdateParamsDocumentsVisa(TypedDict):
        files: NotRequired["List[str]"]
        """
        One or more document ids returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `account_requirement`.
        """

    class UpdateParamsRegisteredAddress(TypedDict):
        city: NotRequired["str"]
        """
        City, district, suburb, town, or village.
        """
        country: NotRequired["str"]
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: NotRequired["str"]
        """
        Address line 1 (e.g., street, PO Box, or company name).
        """
        line2: NotRequired["str"]
        """
        Address line 2 (e.g., apartment, suite, unit, or building).
        """
        postal_code: NotRequired["str"]
        """
        ZIP or postal code.
        """
        state: NotRequired["str"]
        """
        State, county, province, or region.
        """

    class UpdateParamsRelationship(TypedDict):
        director: NotRequired["bool"]
        """
        Whether the person is a director of the account's legal entity. Directors are typically members of the governing board of the company, or responsible for ensuring the company meets its regulatory obligations.
        """
        executive: NotRequired["bool"]
        """
        Whether the person has significant responsibility to control, manage, or direct the organization.
        """
        legal_guardian: NotRequired["bool"]
        """
        Whether the person is the legal guardian of the account's representative.
        """
        owner: NotRequired["bool"]
        """
        Whether the person is an owner of the account's legal entity.
        """
        percent_ownership: NotRequired["Literal['']|float"]
        """
        The percent owned by the person of the account's legal entity.
        """
        representative: NotRequired["bool"]
        """
        Whether the person is authorized as the primary representative of the account. This is the person nominated by the business to provide information about themselves, and general information about the account. There can only be one representative at any given time. At the time the account is created, this person should be set to the person responsible for opening the account.
        """
        title: NotRequired["str"]
        """
        The person's title (e.g., CEO, Support Engineer).
        """

    class UpdateParamsVerification(TypedDict):
        additional_document: NotRequired[
            "AccountPersonService.UpdateParamsVerificationAdditionalDocument"
        ]
        """
        A document showing address, either a passport, local ID card, or utility bill from a well-known utility company.
        """
        document: NotRequired[
            "AccountPersonService.UpdateParamsVerificationDocument"
        ]
        """
        An identifying document, either a passport or local ID card.
        """

    class UpdateParamsVerificationAdditionalDocument(TypedDict):
        back: NotRequired["str"]
        """
        The back of an ID returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `identity_document`. The uploaded file needs to be a color image (smaller than 8,000px by 8,000px), in JPG, PNG, or PDF format, and less than 10 MB in size.
        """
        front: NotRequired["str"]
        """
        The front of an ID returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `identity_document`. The uploaded file needs to be a color image (smaller than 8,000px by 8,000px), in JPG, PNG, or PDF format, and less than 10 MB in size.
        """

    class UpdateParamsVerificationDocument(TypedDict):
        back: NotRequired["str"]
        """
        The back of an ID returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `identity_document`. The uploaded file needs to be a color image (smaller than 8,000px by 8,000px), in JPG, PNG, or PDF format, and less than 10 MB in size.
        """
        front: NotRequired["str"]
        """
        The front of an ID returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `identity_document`. The uploaded file needs to be a color image (smaller than 8,000px by 8,000px), in JPG, PNG, or PDF format, and less than 10 MB in size.
        """

    def delete(
        self,
        account: str,
        person: str,
        params: "AccountPersonService.DeleteParams" = {},
        options: RequestOptions = {},
    ) -> Person:
        """
        Deletes an existing person's relationship to the account's legal entity. Any person with a relationship for an account can be deleted through the API, except if the person is the account_opener. If your integration is using the executive parameter, you cannot delete the only verified executive on file.
        """
        return cast(
            Person,
            self._request(
                "delete",
                "/v1/accounts/{account}/persons/{person}".format(
                    account=sanitize_id(account),
                    person=sanitize_id(person),
                ),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def delete_async(
        self,
        account: str,
        person: str,
        params: "AccountPersonService.DeleteParams" = {},
        options: RequestOptions = {},
    ) -> Person:
        """
        Deletes an existing person's relationship to the account's legal entity. Any person with a relationship for an account can be deleted through the API, except if the person is the account_opener. If your integration is using the executive parameter, you cannot delete the only verified executive on file.
        """
        return cast(
            Person,
            await self._request_async(
                "delete",
                "/v1/accounts/{account}/persons/{person}".format(
                    account=sanitize_id(account),
                    person=sanitize_id(person),
                ),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        account: str,
        person: str,
        params: "AccountPersonService.RetrieveParams" = {},
        options: RequestOptions = {},
    ) -> Person:
        """
        Retrieves an existing person.
        """
        return cast(
            Person,
            self._request(
                "get",
                "/v1/accounts/{account}/persons/{person}".format(
                    account=sanitize_id(account),
                    person=sanitize_id(person),
                ),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        account: str,
        person: str,
        params: "AccountPersonService.RetrieveParams" = {},
        options: RequestOptions = {},
    ) -> Person:
        """
        Retrieves an existing person.
        """
        return cast(
            Person,
            await self._request_async(
                "get",
                "/v1/accounts/{account}/persons/{person}".format(
                    account=sanitize_id(account),
                    person=sanitize_id(person),
                ),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        account: str,
        person: str,
        params: "AccountPersonService.UpdateParams" = {},
        options: RequestOptions = {},
    ) -> Person:
        """
        Updates an existing person.
        """
        return cast(
            Person,
            self._request(
                "post",
                "/v1/accounts/{account}/persons/{person}".format(
                    account=sanitize_id(account),
                    person=sanitize_id(person),
                ),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        account: str,
        person: str,
        params: "AccountPersonService.UpdateParams" = {},
        options: RequestOptions = {},
    ) -> Person:
        """
        Updates an existing person.
        """
        return cast(
            Person,
            await self._request_async(
                "post",
                "/v1/accounts/{account}/persons/{person}".format(
                    account=sanitize_id(account),
                    person=sanitize_id(person),
                ),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def list(
        self,
        account: str,
        params: "AccountPersonService.ListParams" = {},
        options: RequestOptions = {},
    ) -> ListObject[Person]:
        """
        Returns a list of people associated with the account's legal entity. The people are returned sorted by creation date, with the most recent people appearing first.
        """
        return cast(
            ListObject[Person],
            self._request(
                "get",
                "/v1/accounts/{account}/persons".format(
                    account=sanitize_id(account),
                ),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        account: str,
        params: "AccountPersonService.ListParams" = {},
        options: RequestOptions = {},
    ) -> ListObject[Person]:
        """
        Returns a list of people associated with the account's legal entity. The people are returned sorted by creation date, with the most recent people appearing first.
        """
        return cast(
            ListObject[Person],
            await self._request_async(
                "get",
                "/v1/accounts/{account}/persons".format(
                    account=sanitize_id(account),
                ),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        account: str,
        params: "AccountPersonService.CreateParams" = {},
        options: RequestOptions = {},
    ) -> Person:
        """
        Creates a new person.
        """
        return cast(
            Person,
            self._request(
                "post",
                "/v1/accounts/{account}/persons".format(
                    account=sanitize_id(account),
                ),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        account: str,
        params: "AccountPersonService.CreateParams" = {},
        options: RequestOptions = {},
    ) -> Person:
        """
        Creates a new person.
        """
        return cast(
            Person,
            await self._request_async(
                "post",
                "/v1/accounts/{account}/persons".format(
                    account=sanitize_id(account),
                ),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._charge import Charge
from stripe._list_object import ListObject
from stripe._request_options import RequestOptions
from stripe._search_result_object import SearchResultObject
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Dict, List, Union, cast
from typing_extensions import Literal, NotRequired, TypedDict


class ChargeService(StripeService):
    class CaptureParams(TypedDict):
        amount: NotRequired["int"]
        """
        The amount to capture, which must be less than or equal to the original amount. Any additional amount will be automatically refunded.
        """
        application_fee: NotRequired["int"]
        """
        An application fee to add on to this charge.
        """
        application_fee_amount: NotRequired["int"]
        """
        An application fee amount to add on to this charge, which must be less than or equal to the original amount.
        """
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """
        payment_details: NotRequired[
            "ChargeService.CaptureParamsPaymentDetails"
        ]
        """
        Provides industry-specific information about the charge.
        """
        receipt_email: NotRequired["str"]
        """
        The email address to send this charge's receipt to. This will override the previously-specified email address for this charge, if one was set. Receipts will not be sent in test mode.
        """
        statement_descriptor: NotRequired["str"]
        """
        For card charges, use `statement_descriptor_suffix` instead. Otherwise, you can use this value as the complete description of a charge on your customers' statements. Must contain at least one letter, maximum 22 characters.
        """
        statement_descriptor_suffix: NotRequired["str"]
        """
        Provides information about the charge that customers see on their statements. Concatenated with the prefix (shortened descriptor) or statement descriptor that's set on the account to form the complete statement descriptor. Maximum 22 characters for the concatenated descriptor.
        """
        transfer_data: NotRequired["ChargeService.CaptureParamsTransferData"]
        """
        An optional dictionary including the account to automatically transfer to as part of a destination charge. [See the Connect documentation](https://stripe.com/docs/connect/destination-charges) for details.
        """
        transfer_group: NotRequired["str"]
        """
        A string that identifies this transaction as part of a group. `transfer_group` may only be provided if it has not been set. See the [Connect documentation](https://stripe.com/docs/connect/separate-charges-and-transfers#transfer-options) for details.
        """

    class CaptureParamsPaymentDetails(TypedDict):
        car_rental: NotRequired[
            "ChargeService.CaptureParamsPaymentDetailsCarRental"
        ]
        """
        Car rental details for this PaymentIntent.
        """
        event_details: NotRequired[
            "ChargeService.CaptureParamsPaymentDetailsEventDetails"
        ]
        """
        Event details for this PaymentIntent
        """
        flight: NotRequired["ChargeService.CaptureParamsPaymentDetailsFlight"]
        """
        Flight reservation details for this PaymentIntent
        """
        lodging: NotRequired[
            "ChargeService.CaptureParamsPaymentDetailsLodging"
        ]
        """
        Lodging reservation details for this PaymentIntent
        """
        subscription: NotRequired[
            "ChargeService.CaptureParamsPaymentDetailsSubscription"
        ]
        """
        Subscription details for this PaymentIntent
        """

    class CaptureParamsPaymentDetailsCarRental(TypedDict):
        affiliate: NotRequired[
            "ChargeService.CaptureParamsPaymentDetailsCarRentalAffiliate"
        ]
        """
        Affiliate details for this purchase.
        """
        booking_number: str
        """
        The booking number associated with the car rental.
        """
        car_class_code: NotRequired["str"]
        """
        Class code of the car.
        """
        car_make: NotRequired["str"]
        """
        Make of the car.
        """
        car_model: NotRequired["str"]
        """
        Model of the car.
        """
        company: NotRequired["str"]
        """
        The name of the rental car company.
        """
        customer_service_phone_number: NotRequired["str"]
        """
        The customer service phone number of the car rental company.
        """
        days_rented: int
        """
        Number of days the car is being rented.
        """
        delivery: NotRequired[
            "ChargeService.CaptureParamsPaymentDetailsCarRentalDelivery"
        ]
        """
        Delivery details for this purchase.
        """
        drivers: NotRequired[
            "List[ChargeService.CaptureParamsPaymentDetailsCarRentalDriver]"
        ]
        """
        The details of the passengers in the travel reservation
        """
        extra_charges: NotRequired[
            "List[Literal['extra_mileage', 'gas', 'late_return', 'one_way_service', 'parking_violation']]"
        ]
        """
        List of additional charges being billed.
        """
        no_show: NotRequired["bool"]
        """
        Indicates if the customer did not keep nor cancel their booking.
        """
        pickup_address: NotRequired[
            "ChargeService.CaptureParamsPaymentDetailsCarRentalPickupAddress"
        ]
        """
        Car pick-up address.
        """
        pickup_at: int
        """
        Car pick-up time. Measured in seconds since the Unix epoch.
        """
        rate_amount: NotRequired["int"]
        """
        Rental rate.
        """
        rate_interval: NotRequired["Literal['day', 'month', 'week']"]
        """
        The frequency at which the rate amount is applied. One of `day`, `week` or `month`
        """
        renter_name: NotRequired["str"]
        """
        The name of the person or entity renting the car.
        """
        return_address: NotRequired[
            "ChargeService.CaptureParamsPaymentDetailsCarRentalReturnAddress"
        ]
        """
        Car return address.
        """
        return_at: int
        """
        Car return time. Measured in seconds since the Unix epoch.
        """
        tax_exempt: NotRequired["bool"]
        """
        Indicates whether the goods or services are tax-exempt or tax is not collected.
        """

    class CaptureParamsPaymentDetailsCarRentalAffiliate(TypedDict):
        name: str
        """
        The name of the affiliate that originated the purchase.
        """

    class CaptureParamsPaymentDetailsCarRentalDelivery(TypedDict):
        mode: NotRequired["Literal['email', 'phone', 'pickup', 'post']"]
        """
        The delivery method for the payment
        """
        recipient: NotRequired[
            "ChargeService.CaptureParamsPaymentDetailsCarRentalDeliveryRecipient"
        ]
        """
        Details of the recipient.
        """

    class CaptureParamsPaymentDetailsCarRentalDeliveryRecipient(TypedDict):
        email: NotRequired["str"]
        """
        The email of the recipient the ticket is delivered to.
        """
        name: NotRequired["str"]
        """
        The name of the recipient the ticket is delivered to.
        """
        phone: NotRequired["str"]
        """
        The phone number of the recipient the ticket is delivered to.
        """

    class CaptureParamsPaymentDetailsCarRentalDriver(TypedDict):
        name: str
        """
        Full name of the person or entity on the car reservation.
        """

    class CaptureParamsPaymentDetailsCarRentalPickupAddress(TypedDict):
        city: NotRequired["str"]
        """
        City, district, suburb, town, or village.
        """
        country: NotRequired["str"]
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: NotRequired["str"]
        """
        Address line 1 (e.g., street, PO Box, or company name).
        """
        line2: NotRequired["str"]
        """
        Address line 2 (e.g., apartment, suite, unit, or building).
        """
        postal_code: NotRequired["str"]
        """
        ZIP or postal code.
        """
        state: NotRequired["str"]
        """
        State, county, province, or region.
        """

    class CaptureParamsPaymentDetailsCarRentalReturnAddress(TypedDict):
        city: NotRequired["str"]
        """
        City, district, suburb, town, or village.
        """
        country: NotRequired["str"]
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: NotRequired["str"]
        """
        Address line 1 (e.g., street, PO Box, or company name).
        """
        line2: NotRequired["str"]
        """
        Address line 2 (e.g., apartment, suite, unit, or building).
        """
        postal_code: NotRequired["str"]
        """
        ZIP or postal code.
        """
        state: NotRequired["str"]
        """
        State, county, province, or region.
        """

    class CaptureParamsPaymentDetailsEventDetails(TypedDict):
        access_controlled_venue: NotRequired["bool"]
        """
        Indicates if the tickets are digitally checked when entering the venue.
        """
        address: NotRequired[
            "ChargeService.CaptureParamsPaymentDetailsEventDetailsAddress"
        ]
        """
        The event location's address.
        """
        affiliate: NotRequired[
            "ChargeService.CaptureParamsPaymentDetailsEventDetailsAffiliate"
        ]
        """
        Affiliate details for this purchase.
        """
        company: NotRequired["str"]
        """
        The name of the company
        """
        delivery: NotRequired[
            "ChargeService.CaptureParamsPaymentDetailsEventDetailsDelivery"
        ]
        """
        Delivery details for this purchase.
        """
        ends_at: NotRequired["int"]
        """
        Event end time. Measured in seconds since the Unix epoch.
        """
        genre: NotRequired["str"]
        """
        Type of the event entertainment (concert, sports event etc)
        """
        name: str
        """
        The name of the event.
        """
        starts_at: NotRequired["int"]
        """
        Event start time. Measured in seconds since the Unix epoch.
        """

    class CaptureParamsPaymentDetailsEventDetailsAddress(TypedDict):
        city: NotRequired["str"]
        """
        City, district, suburb, town, or village.
        """
        country: NotRequired["str"]
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: NotRequired["str"]
        """
        Address line 1 (e.g., street, PO Box, or company name).
        """
        line2: NotRequired["str"]
        """
        Address line 2 (e.g., apartment, suite, unit, or building).
        """
        postal_code: NotRequired["str"]
        """
        ZIP or postal code.
        """
        state: NotRequired["str"]
        """
        State, county, province, or region.
        """

    class CaptureParamsPaymentDetailsEventDetailsAffiliate(TypedDict):
        name: str
        """
        The name of the affiliate that originated the purchase.
        """

    class CaptureParamsPaymentDetailsEventDetailsDelivery(TypedDict):
        mode: NotRequired["Literal['email', 'phone', 'pickup', 'post']"]
        """
        The delivery method for the payment
        """
        recipient: NotRequired[
            "ChargeService.CaptureParamsPaymentDetailsEventDetailsDeliveryRecipient"
        ]
        """
        Details of the recipient.
        """

    class CaptureParamsPaymentDetailsEventDetailsDeliveryRecipient(TypedDict):
        email: NotRequired["str"]
        """
        The email of the recipient the ticket is delivered to.
        """
        name: NotRequired["str"]
        """
        The name of the recipient the ticket is delivered to.
        """
        phone: NotRequired["str"]
        """
        The phone number of the recipient the ticket is delivered to.
        """

    class CaptureParamsPaymentDetailsFlight(TypedDict):
        affiliate: NotRequired[
            "ChargeService.CaptureParamsPaymentDetailsFlightAffiliate"
        ]
        """
        Affiliate details for this purchase.
        """
        agency_number: NotRequired["str"]
        """
        The agency number (i.e. International Air Transport Association (IATA) agency number) of the travel agency that made the booking.
        """
        carrier: NotRequired["str"]
        """
        The International Air Transport Association (IATA) carrier code of the carrier that issued the ticket.
        """
        delivery: NotRequired[
            "ChargeService.CaptureParamsPaymentDetailsFlightDelivery"
        ]
        """
        Delivery details for this purchase.
        """
        passenger_name: NotRequired["str"]
        """
        The name of the person or entity on the reservation.
        """
        passengers: NotRequired[
            "List[ChargeService.CaptureParamsPaymentDetailsFlightPassenger]"
        ]
        """
        The details of the passengers in the travel reservation.
        """
        segments: List[
            "ChargeService.CaptureParamsPaymentDetailsFlightSegment"
        ]
        """
        The individual flight segments associated with the trip.
        """
        ticket_number: NotRequired["str"]
        """
        The ticket number associated with the travel reservation.
        """

    class CaptureParamsPaymentDetailsFlightAffiliate(TypedDict):
        name: str
        """
        The name of the affiliate that originated the purchase.
        """

    class CaptureParamsPaymentDetailsFlightDelivery(TypedDict):
        mode: NotRequired["Literal['email', 'phone', 'pickup', 'post']"]
        """
        The delivery method for the payment
        """
        recipient: NotRequired[
            "ChargeService.CaptureParamsPaymentDetailsFlightDeliveryRecipient"
        ]
        """
        Details of the recipient.
        """

    class CaptureParamsPaymentDetailsFlightDeliveryRecipient(TypedDict):
        email: NotRequired["str"]
        """
        The email of the recipient the ticket is delivered to.
        """
        name: NotRequired["str"]
        """
        The name of the recipient the ticket is delivered to.
        """
        phone: NotRequired["str"]
        """
        The phone number of the recipient the ticket is delivered to.
        """

    class CaptureParamsPaymentDetailsFlightPassenger(TypedDict):
        name: str
        """
        Full name of the person or entity on the flight reservation.
        """

    class CaptureParamsPaymentDetailsFlightSegment(TypedDict):
        amount: NotRequired["int"]
        """
        The flight segment amount.
        """
        arrival_airport: NotRequired["str"]
        """
        The International Air Transport Association (IATA) airport code for the arrival airport.
        """
        arrives_at: NotRequired["int"]
        """
        The arrival time for the flight segment. Measured in seconds since the Unix epoch.
        """
        carrier: NotRequired["str"]
        """
        The International Air Transport Association (IATA) carrier code of the carrier operating the flight segment.
        """
        departs_at: int
        """
        The departure time for the flight segment. Measured in seconds since the Unix epoch.
        """
        departure_airport: NotRequired["str"]
        """
        The International Air Transport Association (IATA) airport code for the departure airport.
        """
        flight_number: NotRequired["str"]
        """
        The flight number associated with the segment
        """
        service_class: NotRequired[
            "Literal['business', 'economy', 'first', 'premium_economy']"
        ]
        """
        The fare class for the segment.
        """

    class CaptureParamsPaymentDetailsLodging(TypedDict):
        address: NotRequired[
            "ChargeService.CaptureParamsPaymentDetailsLodgingAddress"
        ]
        """
        The lodging location's address.
        """
        adults: NotRequired["int"]
        """
        The number of adults on the booking
        """
        affiliate: NotRequired[
            "ChargeService.CaptureParamsPaymentDetailsLodgingAffiliate"
        ]
        """
        Affiliate details for this purchase.
        """
        booking_number: NotRequired["str"]
        """
        The booking number associated with the lodging reservation.
        """
        category: NotRequired["Literal['hotel', 'vacation_rental']"]
        """
        The lodging category
        """
        checkin_at: int
        """
        Loding check-in time. Measured in seconds since the Unix epoch.
        """
        checkout_at: int
        """
        Lodging check-out time. Measured in seconds since the Unix epoch.
        """
        customer_service_phone_number: NotRequired["str"]
        """
        The customer service phone number of the lodging company.
        """
        daily_room_rate_amount: NotRequired["int"]
        """
        The daily lodging room rate.
        """
        delivery: NotRequired[
            "ChargeService.CaptureParamsPaymentDetailsLodgingDelivery"
        ]
        """
        Delivery details for this purchase.
        """
        extra_charges: NotRequired[
            "List[Literal['gift_shop', 'laundry', 'mini_bar', 'other', 'restaurant', 'telephone']]"
        ]
        """
        List of additional charges being billed.
        """
        fire_safety_act_compliance: NotRequired["bool"]
        """
        Indicates whether the lodging location is compliant with the Fire Safety Act.
        """
        name: NotRequired["str"]
        """
        The name of the lodging location.
        """
        no_show: NotRequired["bool"]
        """
        Indicates if the customer did not keep their booking while failing to cancel the reservation.
        """
        number_of_rooms: NotRequired["int"]
        """
        The number of rooms on the booking
        """
        passengers: NotRequired[
            "List[ChargeService.CaptureParamsPaymentDetailsLodgingPassenger]"
        ]
        """
        The details of the passengers in the travel reservation
        """
        property_phone_number: NotRequired["str"]
        """
        The phone number of the lodging location.
        """
        room_class: NotRequired["str"]
        """
        The room class for this purchase.
        """
        room_nights: NotRequired["int"]
        """
        The number of room nights
        """
        total_room_tax_amount: NotRequired["int"]
        """
        The total tax amount associating with the room reservation.
        """
        total_tax_amount: NotRequired["int"]
        """
        The total tax amount
        """

    class CaptureParamsPaymentDetailsLodgingAddress(TypedDict):
        city: NotRequired["str"]
        """
        City, district, suburb, town, or village.
        """
        country: NotRequired["str"]
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: NotRequired["str"]
        """
        Address line 1 (e.g., street, PO Box, or company name).
        """
        line2: NotRequired["str"]
        """
        Address line 2 (e.g., apartment, suite, unit, or building).
        """
        postal_code: NotRequired["str"]
        """
        ZIP or postal code.
        """
        state: NotRequired["str"]
        """
        State, county, province, or region.
        """

    class CaptureParamsPaymentDetailsLodgingAffiliate(TypedDict):
        name: str
        """
        The name of the affiliate that originated the purchase.
        """

    class CaptureParamsPaymentDetailsLodgingDelivery(TypedDict):
        mode: NotRequired["Literal['email', 'phone', 'pickup', 'post']"]
        """
        The delivery method for the payment
        """
        recipient: NotRequired[
            "ChargeService.CaptureParamsPaymentDetailsLodgingDeliveryRecipient"
        ]
        """
        Details of the recipient.
        """

    class CaptureParamsPaymentDetailsLodgingDeliveryRecipient(TypedDict):
        email: NotRequired["str"]
        """
        The email of the recipient the ticket is delivered to.
        """
        name: NotRequired["str"]
        """
        The name of the recipient the ticket is delivered to.
        """
        phone: NotRequired["str"]
        """
        The phone number of the recipient the ticket is delivered to.
        """

    class CaptureParamsPaymentDetailsLodgingPassenger(TypedDict):
        name: str
        """
        Full name of the person or entity on the lodging reservation.
        """

    class CaptureParamsPaymentDetailsSubscription(TypedDict):
        affiliate: NotRequired[
            "ChargeService.CaptureParamsPaymentDetailsSubscriptionAffiliate"
        ]
        """
        Affiliate details for this purchase.
        """
        auto_renewal: NotRequired["bool"]
        """
        Info whether the subscription will be auto renewed upon expiry.
        """
        billing_interval: NotRequired[
            "ChargeService.CaptureParamsPaymentDetailsSubscriptionBillingInterval"
        ]
        """
        Subscription billing details for this purchase.
        """
        ends_at: NotRequired["int"]
        """
        Subscription end time. Measured in seconds since the Unix epoch.
        """
        name: str
        """
        Name of the product on subscription. e.g. Apple Music Subscription
        """
        starts_at: NotRequired["int"]
        """
        Subscription start time. Measured in seconds since the Unix epoch.
        """

    class CaptureParamsPaymentDetailsSubscriptionAffiliate(TypedDict):
        name: str
        """
        The name of the affiliate that originated the purchase.
        """

    class CaptureParamsPaymentDetailsSubscriptionBillingInterval(TypedDict):
        count: int
        """
        The number of intervals, as an whole number greater than 0. Stripe multiplies this by the interval type to get the overall duration.
        """
        interval: Literal["day", "month", "week", "year"]
        """
        Specifies a type of interval unit. Either `day`, `week`, `month` or `year`.
        """

    class CaptureParamsTransferData(TypedDict):
        amount: NotRequired["int"]
        """
        The amount transferred to the destination account, if specified. By default, the entire charge amount is transferred to the destination account.
        """

    class CreateParams(TypedDict):
        amount: NotRequired["int"]
        """
        Amount intended to be collected by this payment. A positive integer representing how much to charge in the [smallest currency unit](https://stripe.com/docs/currencies#zero-decimal) (e.g., 100 cents to charge $1.00 or 100 to charge ¥100, a zero-decimal currency). The minimum amount is $0.50 US or [equivalent in charge currency](https://stripe.com/docs/currencies#minimum-and-maximum-charge-amounts). The amount value supports up to eight digits (e.g., a value of 99999999 for a USD charge of $999,999.99).
        """
        application_fee: NotRequired["int"]
        application_fee_amount: NotRequired["int"]
        """
        A fee in cents (or local equivalent) that will be applied to the charge and transferred to the application owner's Stripe account. The request must be made with an OAuth key or the `Stripe-Account` header in order to take an application fee. For more information, see the application fees [documentation](https://stripe.com/docs/connect/direct-charges#collecting-fees).
        """
        capture: NotRequired["bool"]
        """
        Whether to immediately capture the charge. Defaults to `true`. When `false`, the charge issues an authorization (or pre-authorization), and will need to be [captured](https://stripe.com/docs/api#capture_charge) later. Uncaptured charges expire after a set number of days (7 by default). For more information, see the [authorizing charges and settling later](https://stripe.com/docs/charges/placing-a-hold) documentation.
        """
        currency: NotRequired["str"]
        """
        Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
        """
        customer: NotRequired["str"]
        """
        The ID of an existing customer that will be charged in this request.
        """
        description: NotRequired["str"]
        """
        An arbitrary string which you can attach to a `Charge` object. It is displayed when in the web interface alongside the charge. Note that if you use Stripe to send automatic email receipts to your customers, your receipt emails will include the `description` of the charge(s) that they are describing.
        """
        destination: NotRequired["ChargeService.CreateParamsDestination"]
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """
        metadata: NotRequired["Literal['']|Dict[str, str]"]
        """
        Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
        """
        on_behalf_of: NotRequired["str"]
        """
        The Stripe account ID for which these funds are intended. Automatically set if you use the `destination` parameter. For details, see [Creating Separate Charges and Transfers](https://stripe.com/docs/connect/separate-charges-and-transfers#on-behalf-of).
        """
        radar_options: NotRequired["ChargeService.CreateParamsRadarOptions"]
        """
        Options to configure Radar. See [Radar Session](https://stripe.com/docs/radar/radar-session) for more information.
        """
        receipt_email: NotRequired["str"]
        """
        The email address to which this charge's [receipt](https://stripe.com/docs/dashboard/receipts) will be sent. The receipt will not be sent until the charge is paid, and no receipts will be sent for test mode charges. If this charge is for a [Customer](https://stripe.com/docs/api/customers/object), the email address specified here will override the customer's email address. If `receipt_email` is specified for a charge in live mode, a receipt will be sent regardless of your [email settings](https://dashboard.stripe.com/account/emails).
        """
        shipping: NotRequired["ChargeService.CreateParamsShipping"]
        """
        Shipping information for the charge. Helps prevent fraud on charges for physical goods.
        """
        source: NotRequired["str"]
        """
        A payment source to be charged. This can be the ID of a [card](https://stripe.com/docs/api#cards) (i.e., credit or debit card), a [bank account](https://stripe.com/docs/api#bank_accounts), a [source](https://stripe.com/docs/api#sources), a [token](https://stripe.com/docs/api#tokens), or a [connected account](https://stripe.com/docs/connect/account-debits#charging-a-connected-account). For certain sources---namely, [cards](https://stripe.com/docs/api#cards), [bank accounts](https://stripe.com/docs/api#bank_accounts), and attached [sources](https://stripe.com/docs/api#sources)---you must also pass the ID of the associated customer.
        """
        statement_descriptor: NotRequired["str"]
        """
        For card charges, use `statement_descriptor_suffix` instead. Otherwise, you can use this value as the complete description of a charge on your customers' statements. Must contain at least one letter, maximum 22 characters.
        """
        statement_descriptor_suffix: NotRequired["str"]
        """
        Provides information about the charge that customers see on their statements. Concatenated with the prefix (shortened descriptor) or statement descriptor that's set on the account to form the complete statement descriptor. Maximum 22 characters for the concatenated descriptor.
        """
        transfer_data: NotRequired["ChargeService.CreateParamsTransferData"]
        """
        An optional dictionary including the account to automatically transfer to as part of a destination charge. [See the Connect documentation](https://stripe.com/docs/connect/destination-charges) for details.
        """
        transfer_group: NotRequired["str"]
        """
        A string that identifies this transaction as part of a group. For details, see [Grouping transactions](https://stripe.com/docs/connect/separate-charges-and-transfers#transfer-options).
        """

    class CreateParamsDestination(TypedDict):
        account: str
        """
        ID of an existing, connected Stripe account.
        """
        amount: NotRequired["int"]
        """
        The amount to transfer to the destination account without creating an `Application Fee` object. Cannot be combined with the `application_fee` parameter. Must be less than or equal to the charge amount.
        """

    class CreateParamsRadarOptions(TypedDict):
        session: NotRequired["str"]
        """
        A [Radar Session](https://stripe.com/docs/radar/radar-session) is a snapshot of the browser metadata and device details that help Radar make more accurate predictions on your payments.
        """

    class CreateParamsShipping(TypedDict):
        address: "ChargeService.CreateParamsShippingAddress"
        """
        Shipping address.
        """
        carrier: NotRequired["str"]
        """
        The delivery service that shipped a physical product, such as Fedex, UPS, USPS, etc.
        """
        name: str
        """
        Recipient name.
        """
        phone: NotRequired["str"]
        """
        Recipient phone (including extension).
        """
        tracking_number: NotRequired["str"]
        """
        The tracking number for a physical product, obtained from the delivery service. If multiple tracking numbers were generated for this purchase, please separate them with commas.
        """

    class CreateParamsShippingAddress(TypedDict):
        city: NotRequired["str"]
        """
        City, district, suburb, town, or village.
        """
        country: NotRequired["str"]
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: NotRequired["str"]
        """
        Address line 1 (e.g., street, PO Box, or company name).
        """
        line2: NotRequired["str"]
        """
        Address line 2 (e.g., apartment, suite, unit, or building).
        """
        postal_code: NotRequired["str"]
        """
        ZIP or postal code.
        """
        state: NotRequired["str"]
        """
        State, county, province, or region.
        """

    class CreateParamsTransferData(TypedDict):
        amount: NotRequired["int"]
        """
        The amount transferred to the destination account, if specified. By default, the entire charge amount is transferred to the destination account.
        """
        destination: str
        """
        ID of an existing, connected Stripe account.
        """

    class ListParams(TypedDict):
        created: NotRequired["ChargeService.ListParamsCreated|int"]
        """
        Only return charges that were created during the given date interval.
        """
        customer: NotRequired["str"]
        """
        Only return charges for the customer specified by this customer ID.
        """
        ending_before: NotRequired["str"]
        """
        A cursor for use in pagination. `ending_before` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, starting with `obj_bar`, your subsequent call can include `ending_before=obj_bar` in order to fetch the previous page of the list.
        """
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """
        limit: NotRequired["int"]
        """
        A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
        """
        payment_intent: NotRequired["str"]
        """
        Only return charges that were created by the PaymentIntent specified by this PaymentIntent ID.
        """
        starting_after: NotRequired["str"]
        """
        A cursor for use in pagination. `starting_after` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with `obj_foo`, your subsequent call can include `starting_after=obj_foo` in order to fetch the next page of the list.
        """
        transfer_group: NotRequired["str"]
        """
        Only return charges for this transfer group.
        """

    class ListParamsCreated(TypedDict):
        gt: NotRequired["int"]
        """
        Minimum value to filter by (exclusive)
        """
        gte: NotRequired["int"]
        """
        Minimum value to filter by (inclusive)
        """
        lt: NotRequired["int"]
        """
        Maximum value to filter by (exclusive)
        """
        lte: NotRequired["int"]
        """
        Maximum value to filter by (inclusive)
        """

    class RetrieveParams(TypedDict):
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """

    class SearchParams(TypedDict):
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """
        limit: NotRequired["int"]
        """
        A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
        """
        page: NotRequired["str"]
        """
        A cursor for pagination across multiple pages of results. Don't include this parameter on the first call. Use the next_page value returned in a previous response to request subsequent results.
        """
        query: str
        """
        The search query string. See [search query language](https://stripe.com/docs/search#search-query-language) and the list of supported [query fields for charges](https://stripe.com/docs/search#query-fields-for-charges).
        """

    class UpdateParams(TypedDict):
        customer: NotRequired["str"]
        """
        The ID of an existing customer that will be associated with this request. This field may only be updated if there is no existing associated customer with this charge.
        """
        description: NotRequired["str"]
        """
        An arbitrary string which you can attach to a charge object. It is displayed when in the web interface alongside the charge. Note that if you use Stripe to send automatic email receipts to your customers, your receipt emails will include the `description` of the charge(s) that they are describing.
        """
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """
        fraud_details: NotRequired["ChargeService.UpdateParamsFraudDetails"]
        """
        A set of key-value pairs you can attach to a charge giving information about its riskiness. If you believe a charge is fraudulent, include a `user_report` key with a value of `fraudulent`. If you believe a charge is safe, include a `user_report` key with a value of `safe`. Stripe will use the information you send to improve our fraud detection algorithms.
        """
        metadata: NotRequired["Literal['']|Dict[str, str]"]
        """
        Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
        """
        payment_details: NotRequired[
            "ChargeService.UpdateParamsPaymentDetails"
        ]
        """
        Provides industry-specific information about the charge.
        """
        receipt_email: NotRequired["str"]
        """
        This is the email address that the receipt for this charge will be sent to. If this field is updated, then a new email receipt will be sent to the updated address.
        """
        shipping: NotRequired["ChargeService.UpdateParamsShipping"]
        """
        Shipping information for the charge. Helps prevent fraud on charges for physical goods.
        """
        transfer_group: NotRequired["str"]
        """
        A string that identifies this transaction as part of a group. `transfer_group` may only be provided if it has not been set. See the [Connect documentation](https://stripe.com/docs/connect/separate-charges-and-transfers#transfer-options) for details.
        """

    class UpdateParamsFraudDetails(TypedDict):
        user_report: Union[Literal[""], Literal["fraudulent", "safe"]]
        """
        Either `safe` or `fraudulent`.
        """

    class UpdateParamsPaymentDetails(TypedDict):
        car_rental: NotRequired[
            "ChargeService.UpdateParamsPaymentDetailsCarRental"
        ]
        """
        Car rental details for this PaymentIntent.
        """
        event_details: NotRequired[
            "ChargeService.UpdateParamsPaymentDetailsEventDetails"
        ]
        """
        Event details for this PaymentIntent
        """
        flight: NotRequired["ChargeService.UpdateParamsPaymentDetailsFlight"]
        """
        Flight reservation details for this PaymentIntent
        """
        lodging: NotRequired["ChargeService.UpdateParamsPaymentDetailsLodging"]
        """
        Lodging reservation details for this PaymentIntent
        """
        subscription: NotRequired[
            "ChargeService.UpdateParamsPaymentDetailsSubscription"
        ]
        """
        Subscription details for this PaymentIntent
        """

    class UpdateParamsPaymentDetailsCarRental(TypedDict):
        affiliate: NotRequired[
            "ChargeService.UpdateParamsPaymentDetailsCarRentalAffiliate"
        ]
        """
        Affiliate details for this purchase.
        """
        booking_number: str
        """
        The booking number associated with the car rental.
        """
        car_class_code: NotRequired["str"]
        """
        Class code of the car.
        """
        car_make: NotRequired["str"]
        """
        Make of the car.
        """
        car_model: NotRequired["str"]
        """
        Model of the car.
        """
        company: NotRequired["str"]
        """
        The name of the rental car company.
        """
        customer_service_phone_number: NotRequired["str"]
        """
        The customer service phone number of the car rental company.
        """
        days_rented: int
        """
        Number of days the car is being rented.
        """
        delivery: NotRequired[
            "ChargeService.UpdateParamsPaymentDetailsCarRentalDelivery"
        ]
        """
        Delivery details for this purchase.
        """
        drivers: NotRequired[
            "List[ChargeService.UpdateParamsPaymentDetailsCarRentalDriver]"
        ]
        """
        The details of the passengers in the travel reservation
        """
        extra_charges: NotRequired[
            "List[Literal['extra_mileage', 'gas', 'late_return', 'one_way_service', 'parking_violation']]"
        ]
        """
        List of additional charges being billed.
        """
        no_show: NotRequired["bool"]
        """
        Indicates if the customer did not keep nor cancel their booking.
        """
        pickup_address: NotRequired[
            "ChargeService.UpdateParamsPaymentDetailsCarRentalPickupAddress"
        ]
        """
        Car pick-up address.
        """
        pickup_at: int
        """
        Car pick-up time. Measured in seconds since the Unix epoch.
        """
        rate_amount: NotRequired["int"]
        """
        Rental rate.
        """
        rate_interval: NotRequired["Literal['day', 'month', 'week']"]
        """
        The frequency at which the rate amount is applied. One of `day`, `week` or `month`
        """
        renter_name: NotRequired["str"]
        """
        The name of the person or entity renting the car.
        """
        return_address: NotRequired[
            "ChargeService.UpdateParamsPaymentDetailsCarRentalReturnAddress"
        ]
        """
        Car return address.
        """
        return_at: int
        """
        Car return time. Measured in seconds since the Unix epoch.
        """
        tax_exempt: NotRequired["bool"]
        """
        Indicates whether the goods or services are tax-exempt or tax is not collected.
        """

    class UpdateParamsPaymentDetailsCarRentalAffiliate(TypedDict):
        name: str
        """
        The name of the affiliate that originated the purchase.
        """

    class UpdateParamsPaymentDetailsCarRentalDelivery(TypedDict):
        mode: NotRequired["Literal['email', 'phone', 'pickup', 'post']"]
        """
        The delivery method for the payment
        """
        recipient: NotRequired[
            "ChargeService.UpdateParamsPaymentDetailsCarRentalDeliveryRecipient"
        ]
        """
        Details of the recipient.
        """

    class UpdateParamsPaymentDetailsCarRentalDeliveryRecipient(TypedDict):
        email: NotRequired["str"]
        """
        The email of the recipient the ticket is delivered to.
        """
        name: NotRequired["str"]
        """
        The name of the recipient the ticket is delivered to.
        """
        phone: NotRequired["str"]
        """
        The phone number of the recipient the ticket is delivered to.
        """

    class UpdateParamsPaymentDetailsCarRentalDriver(TypedDict):
        name: str
        """
        Full name of the person or entity on the car reservation.
        """

    class UpdateParamsPaymentDetailsCarRentalPickupAddress(TypedDict):
        city: NotRequired["str"]
        """
        City, district, suburb, town, or village.
        """
        country: NotRequired["str"]
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: NotRequired["str"]
        """
        Address line 1 (e.g., street, PO Box, or company name).
        """
        line2: NotRequired["str"]
        """
        Address line 2 (e.g., apartment, suite, unit, or building).
        """
        postal_code: NotRequired["str"]
        """
        ZIP or postal code.
        """
        state: NotRequired["str"]
        """
        State, county, province, or region.
        """

    class UpdateParamsPaymentDetailsCarRentalReturnAddress(TypedDict):
        city: NotRequired["str"]
        """
        City, district, suburb, town, or village.
        """
        country: NotRequired["str"]
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: NotRequired["str"]
        """
        Address line 1 (e.g., street, PO Box, or company name).
        """
        line2: NotRequired["str"]
        """
        Address line 2 (e.g., apartment, suite, unit, or building).
        """
        postal_code: NotRequired["str"]
        """
        ZIP or postal code.
        """
        state: NotRequired["str"]
        """
        State, county, province, or region.
        """

    class UpdateParamsPaymentDetailsEventDetails(TypedDict):
        access_controlled_venue: NotRequired["bool"]
        """
        Indicates if the tickets are digitally checked when entering the venue.
        """
        address: NotRequired[
            "ChargeService.UpdateParamsPaymentDetailsEventDetailsAddress"
        ]
        """
        The event location's address.
        """
        affiliate: NotRequired[
            "ChargeService.UpdateParamsPaymentDetailsEventDetailsAffiliate"
        ]
        """
        Affiliate details for this purchase.
        """
        company: NotRequired["str"]
        """
        The name of the company
        """
        delivery: NotRequired[
            "ChargeService.UpdateParamsPaymentDetailsEventDetailsDelivery"
        ]
        """
        Delivery details for this purchase.
        """
        ends_at: NotRequired["int"]
        """
        Event end time. Measured in seconds since the Unix epoch.
        """
        genre: NotRequired["str"]
        """
        Type of the event entertainment (concert, sports event etc)
        """
        name: str
        """
        The name of the event.
        """
        starts_at: NotRequired["int"]
        """
        Event start time. Measured in seconds since the Unix epoch.
        """

    class UpdateParamsPaymentDetailsEventDetailsAddress(TypedDict):
        city: NotRequired["str"]
        """
        City, district, suburb, town, or village.
        """
        country: NotRequired["str"]
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: NotRequired["str"]
        """
        Address line 1 (e.g., street, PO Box, or company name).
        """
        line2: NotRequired["str"]
        """
        Address line 2 (e.g., apartment, suite, unit, or building).
        """
        postal_code: NotRequired["str"]
        """
        ZIP or postal code.
        """
        state: NotRequired["str"]
        """
        State, county, province, or region.
        """

    class UpdateParamsPaymentDetailsEventDetailsAffiliate(TypedDict):
        name: str
        """
        The name of the affiliate that originated the purchase.
        """

    class UpdateParamsPaymentDetailsEventDetailsDelivery(TypedDict):
        mode: NotRequired["Literal['email', 'phone', 'pickup', 'post']"]
        """
        The delivery method for the payment
        """
        recipient: NotRequired[
            "ChargeService.UpdateParamsPaymentDetailsEventDetailsDeliveryRecipient"
        ]
        """
        Details of the recipient.
        """

    class UpdateParamsPaymentDetailsEventDetailsDeliveryRecipient(TypedDict):
        email: NotRequired["str"]
        """
        The email of the recipient the ticket is delivered to.
        """
        name: NotRequired["str"]
        """
        The name of the recipient the ticket is delivered to.
        """
        phone: NotRequired["str"]
        """
        The phone number of the recipient the ticket is delivered to.
        """

    class UpdateParamsPaymentDetailsFlight(TypedDict):
        affiliate: NotRequired[
            "ChargeService.UpdateParamsPaymentDetailsFlightAffiliate"
        ]
        """
        Affiliate details for this purchase.
        """
        agency_number: NotRequired["str"]
        """
        The agency number (i.e. International Air Transport Association (IATA) agency number) of the travel agency that made the booking.
        """
        carrier: NotRequired["str"]
        """
        The International Air Transport Association (IATA) carrier code of the carrier that issued the ticket.
        """
        delivery: NotRequired[
            "ChargeService.UpdateParamsPaymentDetailsFlightDelivery"
        ]
        """
        Delivery details for this purchase.
        """
        passenger_name: NotRequired["str"]
        """
        The name of the person or entity on the reservation.
        """
        passengers: NotRequired[
            "List[ChargeService.UpdateParamsPaymentDetailsFlightPassenger]"
        ]
        """
        The details of the passengers in the travel reservation.
        """
        segments: List["ChargeService.UpdateParamsPaymentDetailsFlightSegment"]
        """
        The individual flight segments associated with the trip.
        """
        ticket_number: NotRequired["str"]
        """
        The ticket number associated with the travel reservation.
        """

    class UpdateParamsPaymentDetailsFlightAffiliate(TypedDict):
        name: str
        """
        The name of the affiliate that originated the purchase.
        """

    class UpdateParamsPaymentDetailsFlightDelivery(TypedDict):
        mode: NotRequired["Literal['email', 'phone', 'pickup', 'post']"]
        """
        The delivery method for the payment
        """
        recipient: NotRequired[
            "ChargeService.UpdateParamsPaymentDetailsFlightDeliveryRecipient"
        ]
        """
        Details of the recipient.
        """

    class UpdateParamsPaymentDetailsFlightDeliveryRecipient(TypedDict):
        email: NotRequired["str"]
        """
        The email of the recipient the ticket is delivered to.
        """
        name: NotRequired["str"]
        """
        The name of the recipient the ticket is delivered to.
        """
        phone: NotRequired["str"]
        """
        The phone number of the recipient the ticket is delivered to.
        """

    class UpdateParamsPaymentDetailsFlightPassenger(TypedDict):
        name: str
        """
        Full name of the person or entity on the flight reservation.
        """

    class UpdateParamsPaymentDetailsFlightSegment(TypedDict):
        amount: NotRequired["int"]
        """
        The flight segment amount.
        """
        arrival_airport: NotRequired["str"]
        """
        The International Air Transport Association (IATA) airport code for the arrival airport.
        """
        arrives_at: NotRequired["int"]
        """
        The arrival time for the flight segment. Measured in seconds since the Unix epoch.
        """
        carrier: NotRequired["str"]
        """
        The International Air Transport Association (IATA) carrier code of the carrier operating the flight segment.
        """
        departs_at: int
        """
        The departure time for the flight segment. Measured in seconds since the Unix epoch.
        """
        departure_airport: NotRequired["str"]
        """
        The International Air Transport Association (IATA) airport code for the departure airport.
        """
        flight_number: NotRequired["str"]
        """
        The flight number associated with the segment
        """
        service_class: NotRequired[
            "Literal['business', 'economy', 'first', 'premium_economy']"
        ]
        """
        The fare class for the segment.
        """

    class UpdateParamsPaymentDetailsLodging(TypedDict):
        address: NotRequired[
            "ChargeService.UpdateParamsPaymentDetailsLodgingAddress"
        ]
        """
        The lodging location's address.
        """
        adults: NotRequired["int"]
        """
        The number of adults on the booking
        """
        affiliate: NotRequired[
            "ChargeService.UpdateParamsPaymentDetailsLodgingAffiliate"
        ]
        """
        Affiliate details for this purchase.
        """
        booking_number: NotRequired["str"]
        """
        The booking number associated with the lodging reservation.
        """
        category: NotRequired["Literal['hotel', 'vacation_rental']"]
        """
        The lodging category
        """
        checkin_at: int
        """
        Loding check-in time. Measured in seconds since the Unix epoch.
        """
        checkout_at: int
        """
        Lodging check-out time. Measured in seconds since the Unix epoch.
        """
        customer_service_phone_number: NotRequired["str"]
        """
        The customer service phone number of the lodging company.
        """
        daily_room_rate_amount: NotRequired["int"]
        """
        The daily lodging room rate.
        """
        delivery: NotRequired[
            "ChargeService.UpdateParamsPaymentDetailsLodgingDelivery"
        ]
        """
        Delivery details for this purchase.
        """
        extra_charges: NotRequired[
            "List[Literal['gift_shop', 'laundry', 'mini_bar', 'other', 'restaurant', 'telephone']]"
        ]
        """
        List of additional charges being billed.
        """
        fire_safety_act_compliance: NotRequired["bool"]
        """
        Indicates whether the lodging location is compliant with the Fire Safety Act.
        """
        name: NotRequired["str"]
        """
        The name of the lodging location.
        """
        no_show: NotRequired["bool"]
        """
        Indicates if the customer did not keep their booking while failing to cancel the reservation.
        """
        number_of_rooms: NotRequired["int"]
        """
        The number of rooms on the booking
        """
        passengers: NotRequired[
            "List[ChargeService.UpdateParamsPaymentDetailsLodgingPassenger]"
        ]
        """
        The details of the passengers in the travel reservation
        """
        property_phone_number: NotRequired["str"]
        """
        The phone number of the lodging location.
        """
        room_class: NotRequired["str"]
        """
        The room class for this purchase.
        """
        room_nights: NotRequired["int"]
        """
        The number of room nights
        """
        total_room_tax_amount: NotRequired["int"]
        """
        The total tax amount associating with the room reservation.
        """
        total_tax_amount: NotRequired["int"]
        """
        The total tax amount
        """

    class UpdateParamsPaymentDetailsLodgingAddress(TypedDict):
        city: NotRequired["str"]
        """
        City, district, suburb, town, or village.
        """
        country: NotRequired["str"]
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: NotRequired["str"]
        """
        Address line 1 (e.g., street, PO Box, or company name).
        """
        line2: NotRequired["str"]
        """
        Address line 2 (e.g., apartment, suite, unit, or building).
        """
        postal_code: NotRequired["str"]
        """
        ZIP or postal code.
        """
        state: NotRequired["str"]
        """
        State, county, province, or region.
        """

    class UpdateParamsPaymentDetailsLodgingAffiliate(TypedDict):
        name: str
        """
        The name of the affiliate that originated the purchase.
        """

    class UpdateParamsPaymentDetailsLodgingDelivery(TypedDict):
        mode: NotRequired["Literal['email', 'phone', 'pickup', 'post']"]
        """
        The delivery method for the payment
        """
        recipient: NotRequired[
            "ChargeService.UpdateParamsPaymentDetailsLodgingDeliveryRecipient"
        ]
        """
        Details of the recipient.
        """

    class UpdateParamsPaymentDetailsLodgingDeliveryRecipient(TypedDict):
        email: NotRequired["str"]
        """
        The email of the recipient the ticket is delivered to.
        """
        name: NotRequired["str"]
        """
        The name of the recipient the ticket is delivered to.
        """
        phone: NotRequired["str"]
        """
        The phone number of the recipient the ticket is delivered to.
        """

    class UpdateParamsPaymentDetailsLodgingPassenger(TypedDict):
        name: str
        """
        Full name of the person or entity on the lodging reservation.
        """

    class UpdateParamsPaymentDetailsSubscription(TypedDict):
        affiliate: NotRequired[
            "ChargeService.UpdateParamsPaymentDetailsSubscriptionAffiliate"
        ]
        """
        Affiliate details for this purchase.
        """
        auto_renewal: NotRequired["bool"]
        """
        Info whether the subscription will be auto renewed upon expiry.
        """
        billing_interval: NotRequired[
            "ChargeService.UpdateParamsPaymentDetailsSubscriptionBillingInterval"
        ]
        """
        Subscription billing details for this purchase.
        """
        ends_at: NotRequired["int"]
        """
        Subscription end time. Measured in seconds since the Unix epoch.
        """
        name: str
        """
        Name of the product on subscription. e.g. Apple Music Subscription
        """
        starts_at: NotRequired["int"]
        """
        Subscription start time. Measured in seconds since the Unix epoch.
        """

    class UpdateParamsPaymentDetailsSubscriptionAffiliate(TypedDict):
        name: str
        """
        The name of the affiliate that originated the purchase.
        """

    class UpdateParamsPaymentDetailsSubscriptionBillingInterval(TypedDict):
        count: int
        """
        The number of intervals, as an whole number greater than 0. Stripe multiplies this by the interval type to get the overall duration.
        """
        interval: Literal["day", "month", "week", "year"]
        """
        Specifies a type of interval unit. Either `day`, `week`, `month` or `year`.
        """

    class UpdateParamsShipping(TypedDict):
        address: "ChargeService.UpdateParamsShippingAddress"
        """
        Shipping address.
        """
        carrier: NotRequired["str"]
        """
        The delivery service that shipped a physical product, such as Fedex, UPS, USPS, etc.
        """
        name: str
        """
        Recipient name.
        """
        phone: NotRequired["str"]
        """
        Recipient phone (including extension).
        """
        tracking_number: NotRequired["str"]
        """
        The tracking number for a physical product, obtained from the delivery service. If multiple tracking numbers were generated for this purchase, please separate them with commas.
        """

    class UpdateParamsShippingAddress(TypedDict):
        city: NotRequired["str"]
        """
        City, district, suburb, town, or village.
        """
        country: NotRequired["str"]
        """
        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
        """
        line1: NotRequired["str"]
        """
        Address line 1 (e.g., street, PO Box, or company name).
        """
        line2: NotRequired["str"]
        """
        Address line 2 (e.g., apartment, suite, unit, or building).
        """
        postal_code: NotRequired["str"]
        """
        ZIP or postal code.
        """
        state: NotRequired["str"]
        """
        State, county, province, or region.
        """

    def list(
        self,
        params: "ChargeService.ListParams" = {},
        options: RequestOptions = {},
    ) -> ListObject[Charge]:
        """
        Returns a list of charges you've previously created. The charges are returned in sorted order, with the most recent charges appearing first.
        """
        return cast(
            ListObject[Charge],
            self._request(
                "get",
                "/v1/charges",
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: "ChargeService.ListParams" = {},
        options: RequestOptions = {},
    ) -> ListObject[Charge]:
        """
        Returns a list of charges you've previously created. The charges are returned in sorted order, with the most recent charges appearing first.
        """
        return cast(
            ListObject[Charge],
            await self._request_async(
                "get",
                "/v1/charges",
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "ChargeService.CreateParams" = {},
        options: RequestOptions = {},
    ) -> Charge:
        """
        This method is no longer recommended—use the [Payment Intents API](https://stripe.com/docs/api/payment_intents)
        to initiate a new payment instead. Confirmation of the PaymentIntent creates the Charge
        object used to request payment.
        """
        return cast(
            Charge,
            self._request(
                "post",
                "/v1/charges",
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "ChargeService.CreateParams" = {},
        options: RequestOptions = {},
    ) -> Charge:
        """
        This method is no longer recommended—use the [Payment Intents API](https://stripe.com/docs/api/payment_intents)
        to initiate a new payment instead. Confirmation of the PaymentIntent creates the Charge
        object used to request payment.
        """
        return cast(
            Charge,
            await self._request_async(
                "post",
                "/v1/charges",
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        charge: str,
        params: "ChargeService.RetrieveParams" = {},
        options: RequestOptions = {},
    ) -> Charge:
        """
        Retrieves the details of a charge that has previously been created. Supply the unique charge ID that was returned from your previous request, and Stripe will return the corresponding charge information. The same information is returned when creating or refunding the charge.
        """
        return cast(
            Charge,
            self._request(
                "get",
                "/v1/charges/{charge}".format(charge=sanitize_id(charge)),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        charge: str,
        params: "ChargeService.RetrieveParams" = {},
        options: RequestOptions = {},
    ) -> Charge:
        """
        Retrieves the details of a charge that has previously been created. Supply the unique charge ID that was returned from your previous request, and Stripe will return the corresponding charge information. The same information is returned when creating or refunding the charge.
        """
        return cast(
            Charge,
            await self._request_async(
                "get",
                "/v1/charges/{charge}".format(charge=sanitize_id(charge)),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        charge: str,
        params: "ChargeService.UpdateParams" = {},
        options: RequestOptions = {},
    ) -> Charge:
        """
        Updates the specified charge by setting the values of the parameters passed. Any parameters not provided will be left unchanged.
        """
        return cast(
            Charge,
            self._request(
                "post",
                "/v1/charges/{charge}".format(charge=sanitize_id(charge)),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        charge: str,
        params: "ChargeService.UpdateParams" = {},
        options: RequestOptions = {},
    ) -> Charge:
        """
        Updates the specified charge by setting the values of the parameters passed. Any parameters not provided will be left unchanged.
        """
        return cast(
            Charge,
            await self._request_async(
                "post",
                "/v1/charges/{charge}".format(charge=sanitize_id(charge)),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def search(
        self,
        params: "ChargeService.SearchParams",
        options: RequestOptions = {},
    ) -> SearchResultObject[Charge]:
        """
        Search for charges you've previously created using Stripe's [Search Query Language](https://stripe.com/docs/search#search-query-language).
        Don't use search in read-after-write flows where strict consistency is necessary. Under normal operating
        conditions, data is searchable in less than a minute. Occasionally, propagation of new or updated data can be up
        to an hour behind during outages. Search functionality is not available to merchants in India.
        """
        return cast(
            SearchResultObject[Charge],
            self._request(
                "get",
                "/v1/charges/search",
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def search_async(
        self,
        params: "ChargeService.SearchParams",
        options: RequestOptions = {},
    ) -> SearchResultObject[Charge]:
        """
        Search for charges you've previously created using Stripe's [Search Query Language](https://stripe.com/docs/search#search-query-language).
        Don't use search in read-after-write flows where strict consistency is necessary. Under normal operating
        conditions, data is searchable in less than a minute. Occasionally, propagation of new or updated data can be up
        to an hour behind during outages. Search functionality is not available to merchants in India.
        """
        return cast(
            SearchResultObject[Charge],
            await self._request_async(
                "get",
                "/v1/charges/search",
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def capture(
        self,
        charge: str,
        params: "ChargeService.CaptureParams" = {},
        options: RequestOptions = {},
    ) -> Charge:
        """
        Capture the payment of an existing, uncaptured charge that was created with the capture option set to false.

        Uncaptured payments expire a set number of days after they are created ([7 by default](https://stripe.com/docs/charges/placing-a-hold)), after which they are marked as refunded and capture attempts will fail.

        Don't use this method to capture a PaymentIntent-initiated charge. Use [Capture a PaymentIntent](https://stripe.com/docs/api/payment_intents/capture).
        """
        return cast(
            Charge,
            self._request(
                "post",
                "/v1/charges/{charge}/capture".format(
                    charge=sanitize_id(charge),
                ),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def capture_async(
        self,
        charge: str,
        params: "ChargeService.CaptureParams" = {},
        options: RequestOptions = {},
    ) -> Charge:
        """
        Capture the payment of an existing, uncaptured charge that was created with the capture option set to false.

        Uncaptured payments expire a set number of days after they are created ([7 by default](https://stripe.com/docs/charges/placing-a-hold)), after which they are marked as refunded and capture attempts will fail.

        Don't use this method to capture a PaymentIntent-initiated charge. Use [Capture a PaymentIntent](https://stripe.com/docs/api/payment_intents/capture).
        """
        return cast(
            Charge,
            await self._request_async(
                "post",
                "/v1/charges/{charge}/capture".format(
                    charge=sanitize_id(charge),
                ),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._list_object import ListObject
from stripe._request_options import RequestOptions
from stripe._stripe_service import StripeService
from stripe._topup import Topup
from stripe._util import sanitize_id
from typing import Dict, List, cast
from typing_extensions import Literal, NotRequired, TypedDict


class TopupService(StripeService):
    class CancelParams(TypedDict):
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """

    class CreateParams(TypedDict):
        amount: int
        """
        A positive integer representing how much to transfer.
        """
        currency: str
        """
        Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
        """
        description: NotRequired["str"]
        """
        An arbitrary string attached to the object. Often useful for displaying to users.
        """
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """
        metadata: NotRequired["Literal['']|Dict[str, str]"]
        """
        Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
        """
        source: NotRequired["str"]
        """
        The ID of a source to transfer funds from. For most users, this should be left unspecified which will use the bank account that was set up in the dashboard for the specified currency. In test mode, this can be a test bank token (see [Testing Top-ups](https://stripe.com/docs/connect/testing#testing-top-ups)).
        """
        statement_descriptor: NotRequired["str"]
        """
        Extra information about a top-up for the source's bank statement. Limited to 15 ASCII characters.
        """
        transfer_group: NotRequired["str"]
        """
        A string that identifies this top-up as part of a group.
        """

    class ListParams(TypedDict):
        amount: NotRequired["TopupService.ListParamsAmount|int"]
        """
        A positive integer representing how much to transfer.
        """
        created: NotRequired["TopupService.ListParamsCreated|int"]
        """
        A filter on the list, based on the object `created` field. The value can be a string with an integer Unix timestamp, or it can be a dictionary with a number of different query options.
        """
        ending_before: NotRequired["str"]
        """
        A cursor for use in pagination. `ending_before` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, starting with `obj_bar`, your subsequent call can include `ending_before=obj_bar` in order to fetch the previous page of the list.
        """
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """
        limit: NotRequired["int"]
        """
        A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 10.
        """
        starting_after: NotRequired["str"]
        """
        A cursor for use in pagination. `starting_after` is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with `obj_foo`, your subsequent call can include `starting_after=obj_foo` in order to fetch the next page of the list.
        """
        status: NotRequired[
            "Literal['canceled', 'failed', 'pending', 'succeeded']"
        ]
        """
        Only return top-ups that have the given status. One of `canceled`, `failed`, `pending` or `succeeded`.
        """

    class ListParamsAmount(TypedDict):
        gt: NotRequired["int"]
        """
        Minimum value to filter by (exclusive)
        """
        gte: NotRequired["int"]
        """
        Minimum value to filter by (inclusive)
        """
        lt: NotRequired["int"]
        """
        Maximum value to filter by (exclusive)
        """
        lte: NotRequired["int"]
        """
        Maximum value to filter by (inclusive)
        """

    class ListParamsCreated(TypedDict):
        gt: NotRequired["int"]
        """
        Minimum value to filter by (exclusive)
        """
        gte: NotRequired["int"]
        """
        Minimum value to filter by (inclusive)
        """
        lt: NotRequired["int"]
        """
        Maximum value to filter by (exclusive)
        """
        lte: NotRequired["int"]
        """
        Maximum value to filter by (inclusive)
        """

    class RetrieveParams(TypedDict):
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """

    class UpdateParams(TypedDict):
        description: NotRequired["str"]
        """
        An arbitrary string attached to the object. Often useful for displaying to users.
        """
        expand: NotRequired["List[str]"]
        """
        Specifies which fields in the response should be expanded.
        """
        metadata: NotRequired["Literal['']|Dict[str, str]"]
        """
        Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
        """

    def list(
        self,
        params: "TopupService.ListParams" = {},
        options: RequestOptions = {},
    ) -> ListObject[Topup]:
        """
        Returns a list of top-ups.
        """
        return cast(
            ListObject[Topup],
            self._request(
                "get",
                "/v1/topups",
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: "TopupService.ListParams" = {},
        options: RequestOptions = {},
    ) -> ListObject[Topup]:
        """
        Returns a list of top-ups.
        """
        return cast(
            ListObject[Topup],
            await self._request_async(
                "get",
                "/v1/topups",
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self, params: "TopupService.CreateParams", options: RequestOptions = {}
    ) -> Topup:
        """
        Top up the balance of an account
        """
        return cast(
            Topup,
            self._request(
                "post",
                "/v1/topups",
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self, params: "TopupService.CreateParams", options: RequestOptions = {}
    ) -> Topup:
        """
        Top up the balance of an account
        """
        return cast(
            Topup,
            await self._request_async(
                "post",
                "/v1/topups",
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        topup: str,
        params: "TopupService.RetrieveParams" = {},
        options: RequestOptions = {},
    ) -> Topup:
        """
        Retrieves the details of a top-up that has previously been created. Supply the unique top-up ID that was returned from your previous request, and Stripe will return the corresponding top-up information.
        """
        return cast(
            Topup,
            self._request(
                "get",
                "/v1/topups/{topup}".format(topup=sanitize_id(topup)),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        topup: str,
        params: "TopupService.RetrieveParams" = {},
        options: RequestOptions = {},
    ) -> Topup:
        """
        Retrieves the details of a top-up that has previously been created. Supply the unique top-up ID that was returned from your previous request, and Stripe will return the corresponding top-up information.
        """
        return cast(
            Topup,
            await self._request_async(
                "get",
                "/v1/topups/{topup}".format(topup=sanitize_id(topup)),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        topup: str,
        params: "TopupService.UpdateParams" = {},
        options: RequestOptions = {},
    ) -> Topup:
        """
        Updates the metadata of a top-up. Other top-up details are not editable by design.
        """
        return cast(
            Topup,
            self._request(
                "post",
                "/v1/topups/{topup}".format(topup=sanitize_id(topup)),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        topup: str,
        params: "TopupService.UpdateParams" = {},
        options: RequestOptions = {},
    ) -> Topup:
        """
        Updates the metadata of a top-up. Other top-up details are not editable by design.
        """
        return cast(
            Topup,
            await self._request_async(
                "post",
                "/v1/topups/{topup}".format(topup=sanitize_id(topup)),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def cancel(
        self,
        topup: str,
        params: "TopupService.CancelParams" = {},
        options: RequestOptions = {},
    ) -> Topup:
        """
        Cancels a top-up. Only pending top-ups can be canceled.
        """
        return cast(
            Topup,
            self._request(
                "post",
                "/v1/topups/{topup}/cancel".format(topup=sanitize_id(topup)),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def cancel_async(
        self,
        topup: str,
        params: "TopupService.CancelParams" = {},
        options: RequestOptions = {},
    ) -> Topup:
        """
        Cancels a top-up. Only pending top-ups can be canceled.
        """
        return cast(
            Topup,
            await self._request_async(
                "post",
                "/v1/topups/{topup}/cancel".format(topup=sanitize_id(topup)),
                api_mode="V1",
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe.test_helpers._customer_service import CustomerService
from stripe.test_helpers._issuing_service import IssuingService
from stripe.test_helpers._refund_service import RefundService
from stripe.test_helpers._terminal_service import TerminalService
from stripe.test_helpers._test_clock_service import TestClockService
from stripe.test_helpers._treasury_service import TreasuryService


class TestHelpersService(StripeService):
    def __init__(self, requestor):
        super().__init__(requestor)
        self.customers = CustomerService(self._requestor)
        self.issuing = IssuingService(self._requestor)
        self.refunds = RefundService(self._requestor)
        self.terminal = TerminalService(self._requestor)
        self.test_clocks = TestClockService(self._requestor)
        self.treasury = TreasuryService(self._requestor)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe.checkout._session import Session as Session
from stripe.checkout._session_line_item_service import (
    SessionLineItemService as SessionLineItemService,
)
from stripe.checkout._session_service import SessionService as SessionService
